"""
Configure tags for multiple metrics returns "Accepted" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.metrics_api import MetricsApi
from datadog_api_client.v2.model.metric_bulk_configure_tags_type import MetricBulkConfigureTagsType
from datadog_api_client.v2.model.metric_bulk_tag_config_create import MetricBulkTagConfigCreate
from datadog_api_client.v2.model.metric_bulk_tag_config_create_attributes import MetricBulkTagConfigCreateAttributes
from datadog_api_client.v2.model.metric_bulk_tag_config_create_request import MetricBulkTagConfigCreateRequest
from datadog_api_client.v2.model.metric_bulk_tag_config_email_list import MetricBulkTagConfigEmailList
from datadog_api_client.v2.model.metric_bulk_tag_config_tag_name_list import MetricBulkTagConfigTagNameList

# there is a valid "user" in the system
USER_DATA_ATTRIBUTES_EMAIL = environ["USER_DATA_ATTRIBUTES_EMAIL"]

body = MetricBulkTagConfigCreateRequest(
    data=MetricBulkTagConfigCreate(
        attributes=MetricBulkTagConfigCreateAttributes(
            emails=MetricBulkTagConfigEmailList(
                [
                    USER_DATA_ATTRIBUTES_EMAIL,
                ]
            ),
            tags=MetricBulkTagConfigTagNameList(
                [
                    "test",
                    "exampleconfiguretagsformultiplemetricsreturnsacceptedresponse",
                ]
            ),
        ),
        id="system.load.1",
        type=MetricBulkConfigureTagsType.BULK_MANAGE_TAGS,
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = MetricsApi(api_client)
    response = api_instance.create_bulk_tags_metrics_configuration(body=body)

    print(response)



"""
Search pipelines events returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.ci_visibility_pipelines_api import CIVisibilityPipelinesApi
from datadog_api_client.v2.model.ci_app_pipeline_events_request import CIAppPipelineEventsRequest
from datadog_api_client.v2.model.ci_app_pipelines_query_filter import CIAppPipelinesQueryFilter
from datadog_api_client.v2.model.ci_app_query_options import CIAppQueryOptions
from datadog_api_client.v2.model.ci_app_query_page_options import CIAppQueryPageOptions
from datadog_api_client.v2.model.ci_app_sort import CIAppSort

body = CIAppPipelineEventsRequest(
    filter=CIAppPipelinesQueryFilter(
        _from="now-15m",
        query="@ci.provider.name:github AND @ci.status:error",
        to="now",
    ),
    options=CIAppQueryOptions(
        timezone="GMT",
    ),
    page=CIAppQueryPageOptions(
        limit=5,
    ),
    sort=CIAppSort.TIMESTAMP_ASCENDING,
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = CIVisibilityPipelinesApi(api_client)
    response = api_instance.search_ci_app_pipeline_events(body=body)

    print(response)



"""
Get a quick list of events returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.events_api import EventsApi

configuration = Configuration()
configuration.unstable_operations["list_events"] = True
with ApiClient(configuration) as api_client:
    api_instance = EventsApi(api_client)
    response = api_instance.list_events(
        filter_query="datadog-agent",
        filter_from="2020-09-17T11:48:36+01:00",
        filter_to="2020-09-17T12:48:36+01:00",
        page_limit=5,
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class UsageAttributionSupportedMetrics(ModelSimple):
    """
    Supported fields for usage attribution requests (valid requests contain one or more metrics, or `*` for all).

    :param value: Must be one of ["custom_timeseries_usage", "container_usage", "snmp_percentage", "apm_host_usage", "browser_usage", "npm_host_percentage", "infra_host_usage", "custom_timeseries_percentage", "container_percentage", "api_usage", "apm_host_percentage", "infra_host_percentage", "snmp_usage", "browser_percentage", "api_percentage", "npm_host_usage", "lambda_functions_usage", "lambda_functions_percentage", "lambda_invocations_usage", "lambda_invocations_percentage", "fargate_usage", "fargate_percentage", "profiled_host_usage", "profiled_host_percentage", "profiled_container_usage", "profiled_container_percentage", "dbm_hosts_usage", "dbm_hosts_percentage", "dbm_queries_usage", "dbm_queries_percentage", "estimated_indexed_logs_usage", "estimated_indexed_logs_percentage", "estimated_ingested_logs_usage", "estimated_ingested_logs_percentage", "appsec_usage", "appsec_percentage", "estimated_indexed_spans_usage", "estimated_indexed_spans_percentage", "estimated_ingested_spans_usage", "estimated_ingested_spans_percentage", "apm_fargate_usage", "apm_fargate_percentage", "appsec_fargate_usage", "appsec_fargate_percentage", "estimated_rum_usage_attribution_usage", "estimated_rum_usage_attribution_percentage", "*"].
    :type value: str
    """

    allowed_values = {
        "custom_timeseries_usage",
        "container_usage",
        "snmp_percentage",
        "apm_host_usage",
        "browser_usage",
        "npm_host_percentage",
        "infra_host_usage",
        "custom_timeseries_percentage",
        "container_percentage",
        "api_usage",
        "apm_host_percentage",
        "infra_host_percentage",
        "snmp_usage",
        "browser_percentage",
        "api_percentage",
        "npm_host_usage",
        "lambda_functions_usage",
        "lambda_functions_percentage",
        "lambda_invocations_usage",
        "lambda_invocations_percentage",
        "fargate_usage",
        "fargate_percentage",
        "profiled_host_usage",
        "profiled_host_percentage",
        "profiled_container_usage",
        "profiled_container_percentage",
        "dbm_hosts_usage",
        "dbm_hosts_percentage",
        "dbm_queries_usage",
        "dbm_queries_percentage",
        "estimated_indexed_logs_usage",
        "estimated_indexed_logs_percentage",
        "estimated_ingested_logs_usage",
        "estimated_ingested_logs_percentage",
        "appsec_usage",
        "appsec_percentage",
        "estimated_indexed_spans_usage",
        "estimated_indexed_spans_percentage",
        "estimated_ingested_spans_usage",
        "estimated_ingested_spans_percentage",
        "apm_fargate_usage",
        "apm_fargate_percentage",
        "appsec_fargate_usage",
        "appsec_fargate_percentage",
        "estimated_rum_usage_attribution_usage",
        "estimated_rum_usage_attribution_percentage",
        "*",
    }
    CUSTOM_TIMESERIES_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    CONTAINER_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    SNMP_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APM_HOST_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    BROWSER_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    NPM_HOST_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    INFRA_HOST_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    CUSTOM_TIMESERIES_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    CONTAINER_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    API_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APM_HOST_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    INFRA_HOST_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    SNMP_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    BROWSER_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    API_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    NPM_HOST_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    LAMBDA_FUNCTIONS_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    LAMBDA_FUNCTIONS_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    LAMBDA_INVOCATIONS_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    LAMBDA_INVOCATIONS_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    FARGATE_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    FARGATE_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    PROFILED_HOST_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    PROFILED_HOST_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    PROFILED_CONTAINER_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    PROFILED_CONTAINER_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    DBM_HOSTS_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    DBM_HOSTS_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    DBM_QUERIES_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    DBM_QUERIES_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INDEXED_LOGS_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INDEXED_LOGS_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INGESTED_LOGS_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INGESTED_LOGS_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APPSEC_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APPSEC_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INDEXED_SPANS_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INDEXED_SPANS_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INGESTED_SPANS_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_INGESTED_SPANS_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APM_FARGATE_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APM_FARGATE_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APPSEC_FARGATE_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    APPSEC_FARGATE_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_RUM_USAGE_ATTRIBUTION_USAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ESTIMATED_RUM_USAGE_ATTRIBUTION_PERCENTAGE: ClassVar["UsageAttributionSupportedMetrics"]
    ALL: ClassVar["UsageAttributionSupportedMetrics"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


UsageAttributionSupportedMetrics.CUSTOM_TIMESERIES_USAGE = UsageAttributionSupportedMetrics("custom_timeseries_usage")
UsageAttributionSupportedMetrics.CONTAINER_USAGE = UsageAttributionSupportedMetrics("container_usage")
UsageAttributionSupportedMetrics.SNMP_PERCENTAGE = UsageAttributionSupportedMetrics("snmp_percentage")
UsageAttributionSupportedMetrics.APM_HOST_USAGE = UsageAttributionSupportedMetrics("apm_host_usage")
UsageAttributionSupportedMetrics.BROWSER_USAGE = UsageAttributionSupportedMetrics("browser_usage")
UsageAttributionSupportedMetrics.NPM_HOST_PERCENTAGE = UsageAttributionSupportedMetrics("npm_host_percentage")
UsageAttributionSupportedMetrics.INFRA_HOST_USAGE = UsageAttributionSupportedMetrics("infra_host_usage")
UsageAttributionSupportedMetrics.CUSTOM_TIMESERIES_PERCENTAGE = UsageAttributionSupportedMetrics(
    "custom_timeseries_percentage"
)
UsageAttributionSupportedMetrics.CONTAINER_PERCENTAGE = UsageAttributionSupportedMetrics("container_percentage")
UsageAttributionSupportedMetrics.API_USAGE = UsageAttributionSupportedMetrics("api_usage")
UsageAttributionSupportedMetrics.APM_HOST_PERCENTAGE = UsageAttributionSupportedMetrics("apm_host_percentage")
UsageAttributionSupportedMetrics.INFRA_HOST_PERCENTAGE = UsageAttributionSupportedMetrics("infra_host_percentage")
UsageAttributionSupportedMetrics.SNMP_USAGE = UsageAttributionSupportedMetrics("snmp_usage")
UsageAttributionSupportedMetrics.BROWSER_PERCENTAGE = UsageAttributionSupportedMetrics("browser_percentage")
UsageAttributionSupportedMetrics.API_PERCENTAGE = UsageAttributionSupportedMetrics("api_percentage")
UsageAttributionSupportedMetrics.NPM_HOST_USAGE = UsageAttributionSupportedMetrics("npm_host_usage")
UsageAttributionSupportedMetrics.LAMBDA_FUNCTIONS_USAGE = UsageAttributionSupportedMetrics("lambda_functions_usage")
UsageAttributionSupportedMetrics.LAMBDA_FUNCTIONS_PERCENTAGE = UsageAttributionSupportedMetrics(
    "lambda_functions_percentage"
)
UsageAttributionSupportedMetrics.LAMBDA_INVOCATIONS_USAGE = UsageAttributionSupportedMetrics("lambda_invocations_usage")
UsageAttributionSupportedMetrics.LAMBDA_INVOCATIONS_PERCENTAGE = UsageAttributionSupportedMetrics(
    "lambda_invocations_percentage"
)
UsageAttributionSupportedMetrics.FARGATE_USAGE = UsageAttributionSupportedMetrics("fargate_usage")
UsageAttributionSupportedMetrics.FARGATE_PERCENTAGE = UsageAttributionSupportedMetrics("fargate_percentage")
UsageAttributionSupportedMetrics.PROFILED_HOST_USAGE = UsageAttributionSupportedMetrics("profiled_host_usage")
UsageAttributionSupportedMetrics.PROFILED_HOST_PERCENTAGE = UsageAttributionSupportedMetrics("profiled_host_percentage")
UsageAttributionSupportedMetrics.PROFILED_CONTAINER_USAGE = UsageAttributionSupportedMetrics("profiled_container_usage")
UsageAttributionSupportedMetrics.PROFILED_CONTAINER_PERCENTAGE = UsageAttributionSupportedMetrics(
    "profiled_container_percentage"
)
UsageAttributionSupportedMetrics.DBM_HOSTS_USAGE = UsageAttributionSupportedMetrics("dbm_hosts_usage")
UsageAttributionSupportedMetrics.DBM_HOSTS_PERCENTAGE = UsageAttributionSupportedMetrics("dbm_hosts_percentage")
UsageAttributionSupportedMetrics.DBM_QUERIES_USAGE = UsageAttributionSupportedMetrics("dbm_queries_usage")
UsageAttributionSupportedMetrics.DBM_QUERIES_PERCENTAGE = UsageAttributionSupportedMetrics("dbm_queries_percentage")
UsageAttributionSupportedMetrics.ESTIMATED_INDEXED_LOGS_USAGE = UsageAttributionSupportedMetrics(
    "estimated_indexed_logs_usage"
)
UsageAttributionSupportedMetrics.ESTIMATED_INDEXED_LOGS_PERCENTAGE = UsageAttributionSupportedMetrics(
    "estimated_indexed_logs_percentage"
)
UsageAttributionSupportedMetrics.ESTIMATED_INGESTED_LOGS_USAGE = UsageAttributionSupportedMetrics(
    "estimated_ingested_logs_usage"
)
UsageAttributionSupportedMetrics.ESTIMATED_INGESTED_LOGS_PERCENTAGE = UsageAttributionSupportedMetrics(
    "estimated_ingested_logs_percentage"
)
UsageAttributionSupportedMetrics.APPSEC_USAGE = UsageAttributionSupportedMetrics("appsec_usage")
UsageAttributionSupportedMetrics.APPSEC_PERCENTAGE = UsageAttributionSupportedMetrics("appsec_percentage")
UsageAttributionSupportedMetrics.ESTIMATED_INDEXED_SPANS_USAGE = UsageAttributionSupportedMetrics(
    "estimated_indexed_spans_usage"
)
UsageAttributionSupportedMetrics.ESTIMATED_INDEXED_SPANS_PERCENTAGE = UsageAttributionSupportedMetrics(
    "estimated_indexed_spans_percentage"
)
UsageAttributionSupportedMetrics.ESTIMATED_INGESTED_SPANS_USAGE = UsageAttributionSupportedMetrics(
    "estimated_ingested_spans_usage"
)
UsageAttributionSupportedMetrics.ESTIMATED_INGESTED_SPANS_PERCENTAGE = UsageAttributionSupportedMetrics(
    "estimated_ingested_spans_percentage"
)
UsageAttributionSupportedMetrics.APM_FARGATE_USAGE = UsageAttributionSupportedMetrics("apm_fargate_usage")
UsageAttributionSupportedMetrics.APM_FARGATE_PERCENTAGE = UsageAttributionSupportedMetrics("apm_fargate_percentage")
UsageAttributionSupportedMetrics.APPSEC_FARGATE_USAGE = UsageAttributionSupportedMetrics("appsec_fargate_usage")
UsageAttributionSupportedMetrics.APPSEC_FARGATE_PERCENTAGE = UsageAttributionSupportedMetrics(
    "appsec_fargate_percentage"
)
UsageAttributionSupportedMetrics.ESTIMATED_RUM_USAGE_ATTRIBUTION_USAGE = UsageAttributionSupportedMetrics(
    "estimated_rum_usage_attribution_usage"
)
UsageAttributionSupportedMetrics.ESTIMATED_RUM_USAGE_ATTRIBUTION_PERCENTAGE = UsageAttributionSupportedMetrics(
    "estimated_rum_usage_attribution_percentage"
)
UsageAttributionSupportedMetrics.ALL = UsageAttributionSupportedMetrics("*")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.partial_application_key import PartialApplicationKey
    from datadog_api_client.v2.model.application_key_response_included_item import ApplicationKeyResponseIncludedItem
    from datadog_api_client.v2.model.user import User
    from datadog_api_client.v2.model.role import Role


class ListApplicationKeysResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.partial_application_key import PartialApplicationKey
        from datadog_api_client.v2.model.application_key_response_included_item import (
            ApplicationKeyResponseIncludedItem,
        )

        return {
            "data": ([PartialApplicationKey],),
            "included": ([ApplicationKeyResponseIncludedItem],),
        }

    attribute_map = {
        "data": "data",
        "included": "included",
    }

    def __init__(
        self_,
        data: Union[List[PartialApplicationKey], UnsetType] = unset,
        included: Union[List[Union[ApplicationKeyResponseIncludedItem, User, Role]], UnsetType] = unset,
        **kwargs,
    ):
        """
        Response for a list of application keys.

        :param data: Array of application keys.
        :type data: [PartialApplicationKey], optional

        :param included: Array of objects related to the application key.
        :type included: [ApplicationKeyResponseIncludedItem], optional
        """
        if data is not unset:
            kwargs["data"] = data
        if included is not unset:
            kwargs["included"] = included
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetImageSizing(ModelSimple):
    """
    How to size the image on the widget. The values are based on the image `object-fit` CSS properties.
        **Note**: `zoom`, `fit` and `center` values are deprecated.

    :param value: Must be one of ["fill", "contain", "cover", "none", "scale-down", "zoom", "fit", "center"].
    :type value: str
    """

    allowed_values = {
        "fill",
        "contain",
        "cover",
        "none",
        "scale-down",
        "zoom",
        "fit",
        "center",
    }
    FILL: ClassVar["WidgetImageSizing"]
    CONTAIN: ClassVar["WidgetImageSizing"]
    COVER: ClassVar["WidgetImageSizing"]
    NONE: ClassVar["WidgetImageSizing"]
    SCALEDOWN: ClassVar["WidgetImageSizing"]
    ZOOM: ClassVar["WidgetImageSizing"]
    FIT: ClassVar["WidgetImageSizing"]
    CENTER: ClassVar["WidgetImageSizing"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetImageSizing.FILL = WidgetImageSizing("fill")
WidgetImageSizing.CONTAIN = WidgetImageSizing("contain")
WidgetImageSizing.COVER = WidgetImageSizing("cover")
WidgetImageSizing.NONE = WidgetImageSizing("none")
WidgetImageSizing.SCALEDOWN = WidgetImageSizing("scale-down")
WidgetImageSizing.ZOOM = WidgetImageSizing("zoom")
WidgetImageSizing.FIT = WidgetImageSizing("fit")
WidgetImageSizing.CENTER = WidgetImageSizing("center")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.relationship_to_organization import RelationshipToOrganization
    from datadog_api_client.v2.model.relationship_to_organizations import RelationshipToOrganizations
    from datadog_api_client.v2.model.relationship_to_users import RelationshipToUsers
    from datadog_api_client.v2.model.relationship_to_roles import RelationshipToRoles


class UserResponseRelationships(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.relationship_to_organization import RelationshipToOrganization
        from datadog_api_client.v2.model.relationship_to_organizations import RelationshipToOrganizations
        from datadog_api_client.v2.model.relationship_to_users import RelationshipToUsers
        from datadog_api_client.v2.model.relationship_to_roles import RelationshipToRoles

        return {
            "org": (RelationshipToOrganization,),
            "other_orgs": (RelationshipToOrganizations,),
            "other_users": (RelationshipToUsers,),
            "roles": (RelationshipToRoles,),
        }

    attribute_map = {
        "org": "org",
        "other_orgs": "other_orgs",
        "other_users": "other_users",
        "roles": "roles",
    }

    def __init__(
        self_,
        org: Union[RelationshipToOrganization, UnsetType] = unset,
        other_orgs: Union[RelationshipToOrganizations, UnsetType] = unset,
        other_users: Union[RelationshipToUsers, UnsetType] = unset,
        roles: Union[RelationshipToRoles, UnsetType] = unset,
        **kwargs,
    ):
        """
        Relationships of the user object returned by the API.

        :param org: Relationship to an organization.
        :type org: RelationshipToOrganization, optional

        :param other_orgs: Relationship to organizations.
        :type other_orgs: RelationshipToOrganizations, optional

        :param other_users: Relationship to users.
        :type other_users: RelationshipToUsers, optional

        :param roles: Relationship to roles.
        :type roles: RelationshipToRoles, optional
        """
        if org is not unset:
            kwargs["org"] = org
        if other_orgs is not unset:
            kwargs["other_orgs"] = other_orgs
        if other_users is not unset:
            kwargs["other_users"] = other_users
        if roles is not unset:
            kwargs["roles"] = roles
        super().__init__(kwargs)



"""
Create a new dashboard with alert_value widget
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.alert_value_widget_definition import AlertValueWidgetDefinition
from datadog_api_client.v1.model.alert_value_widget_definition_type import AlertValueWidgetDefinitionType
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_layout import WidgetLayout
from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign

# there is a valid "monitor" in the system
MONITOR_ID = environ["MONITOR_ID"]

body = Dashboard(
    title="Example-Create_a_new_dashboard_with_alert_value_widget",
    description="",
    widgets=[
        Widget(
            layout=WidgetLayout(
                x=0,
                y=0,
                width=15,
                height=8,
            ),
            definition=AlertValueWidgetDefinition(
                title="",
                title_size="16",
                title_align=WidgetTextAlign.LEFT,
                type=AlertValueWidgetDefinitionType.ALERT_VALUE,
                alert_id="7",
                unit="auto",
                text_align=WidgetTextAlign.LEFT,
                precision=2,
            ),
        ),
    ],
    template_variables=[],
    layout_type=DashboardLayoutType.FREE,
    is_read_only=False,
    notify_list=[],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



"""
Create a new dashboard with alert_graph widget
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.alert_graph_widget_definition import AlertGraphWidgetDefinition
from datadog_api_client.v1.model.alert_graph_widget_definition_type import AlertGraphWidgetDefinitionType
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_layout import WidgetLayout
from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign
from datadog_api_client.v1.model.widget_time import WidgetTime
from datadog_api_client.v1.model.widget_viz_type import WidgetVizType

# there is a valid "monitor" in the system
MONITOR_ID = environ["MONITOR_ID"]

body = Dashboard(
    title="Example-Create_a_new_dashboard_with_alert_graph_widget",
    description="",
    widgets=[
        Widget(
            layout=WidgetLayout(
                x=0,
                y=0,
                width=47,
                height=15,
            ),
            definition=AlertGraphWidgetDefinition(
                title="",
                title_size="16",
                title_align=WidgetTextAlign.LEFT,
                time=WidgetTime(),
                type=AlertGraphWidgetDefinitionType.ALERT_GRAPH,
                alert_id="7",
                viz_type=WidgetVizType.TIMESERIES,
            ),
        ),
    ],
    template_variables=[],
    layout_type=DashboardLayoutType.FREE,
    is_read_only=False,
    notify_list=[],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


class SyntheticsTestMetadata(ModelNormal):
    @cached_property
    def additional_properties_type(_):
        return (str,)

    def __init__(self_, **kwargs):
        """
        Metadata to include when performing the gRPC test.
        """
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class ProcessSummariesMetaPage(ModelNormal):
    validations = {
        "size": {
            "inclusive_maximum": 10000,
            "inclusive_minimum": 0,
        },
    }

    @cached_property
    def openapi_types(_):
        return {
            "after": (str,),
            "size": (int,),
        }

    attribute_map = {
        "after": "after",
        "size": "size",
    }

    def __init__(self_, after: Union[str, UnsetType] = unset, size: Union[int, UnsetType] = unset, **kwargs):
        """
        Paging attributes.

        :param after: The cursor used to get the next results, if any. To make the next request, use the same
            parameters with the addition of the ``page[cursor]``.
        :type after: str, optional

        :param size: Number of results returned.
        :type size: int, optional
        """
        if after is not unset:
            kwargs["after"] = after
        if size is not unset:
            kwargs["size"] = size
        super().__init__(kwargs)



"""
Send logs returns "Request accepted for processing (always 202 empty JSON)." response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.logs_api import LogsApi
from datadog_api_client.v2.model.http_log import HTTPLog
from datadog_api_client.v2.model.http_log_item import HTTPLogItem

body = HTTPLog(
    [
        HTTPLogItem(
            ddsource="nginx",
            ddtags="env:staging,version:5.1",
            hostname="i-012345678",
            message="2019-11-19T14:37:58,995 INFO [process.name][20081] Hello World",
            service="payment",
            status="error",
        ),
    ]
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = LogsApi(api_client)
    response = api_instance.submit_log(body=body)

    print(response)



"""
Delete a tag filtering entry returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.aws_integration_api import AWSIntegrationApi
from datadog_api_client.v1.model.aws_namespace import AWSNamespace
from datadog_api_client.v1.model.aws_tag_filter_delete_request import AWSTagFilterDeleteRequest

body = AWSTagFilterDeleteRequest(
    account_id="FAKEAC0FAKEAC2FAKEAC",
    namespace=AWSNamespace.ELB,
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = AWSIntegrationApi(api_client)
    response = api_instance.delete_aws_tag_filter(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class EventPriority(ModelSimple):
    """
    The priority of the event's monitor. For example, `normal` or `low`.

    :param value: Must be one of ["normal", "low"].
    :type value: str
    """

    allowed_values = {
        "normal",
        "low",
    }
    NORMAL: ClassVar["EventPriority"]
    LOW: ClassVar["EventPriority"]

    _nullable = True

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


EventPriority.NORMAL = EventPriority("normal")
EventPriority.LOW = EventPriority("low")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_trigger_ci_test_location import SyntheticsTriggerCITestLocation
    from datadog_api_client.v1.model.synthetics_trigger_ci_test_run_result import SyntheticsTriggerCITestRunResult


class SyntheticsTriggerCITestsResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_trigger_ci_test_location import SyntheticsTriggerCITestLocation
        from datadog_api_client.v1.model.synthetics_trigger_ci_test_run_result import SyntheticsTriggerCITestRunResult

        return {
            "batch_id": (str, none_type),
            "locations": ([SyntheticsTriggerCITestLocation],),
            "results": ([SyntheticsTriggerCITestRunResult],),
            "triggered_check_ids": ([str],),
        }

    attribute_map = {
        "batch_id": "batch_id",
        "locations": "locations",
        "results": "results",
        "triggered_check_ids": "triggered_check_ids",
    }

    def __init__(
        self_,
        batch_id: Union[str, none_type, UnsetType] = unset,
        locations: Union[List[SyntheticsTriggerCITestLocation], UnsetType] = unset,
        results: Union[List[SyntheticsTriggerCITestRunResult], UnsetType] = unset,
        triggered_check_ids: Union[List[str], UnsetType] = unset,
        **kwargs,
    ):
        """
        Object containing information about the tests triggered.

        :param batch_id: The public ID of the batch triggered.
        :type batch_id: str, none_type, optional

        :param locations: List of Synthetics locations.
        :type locations: [SyntheticsTriggerCITestLocation], optional

        :param results: Information about the tests runs.
        :type results: [SyntheticsTriggerCITestRunResult], optional

        :param triggered_check_ids: The public IDs of the Synthetics test triggered.
        :type triggered_check_ids: [str], optional
        """
        if batch_id is not unset:
            kwargs["batch_id"] = batch_id
        if locations is not unset:
            kwargs["locations"] = locations
        if results is not unset:
            kwargs["results"] = results
        if triggered_check_ids is not unset:
            kwargs["triggered_check_ids"] = triggered_check_ids
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class MetricContentEncoding(ModelSimple):
    """
    HTTP header used to compress the media-type.

    :param value: If omitted defaults to "deflate". Must be one of ["deflate", "zstd1", "gzip"].
    :type value: str
    """

    allowed_values = {
        "deflate",
        "zstd1",
        "gzip",
    }
    DEFLATE: ClassVar["MetricContentEncoding"]
    ZSTD1: ClassVar["MetricContentEncoding"]
    GZIP: ClassVar["MetricContentEncoding"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


MetricContentEncoding.DEFLATE = MetricContentEncoding("deflate")
MetricContentEncoding.ZSTD1 = MetricContentEncoding("zstd1")
MetricContentEncoding.GZIP = MetricContentEncoding("gzip")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.scalar_formula_response_atrributes import ScalarFormulaResponseAtrributes
    from datadog_api_client.v2.model.scalar_formula_response_type import ScalarFormulaResponseType


class ScalarResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.scalar_formula_response_atrributes import ScalarFormulaResponseAtrributes
        from datadog_api_client.v2.model.scalar_formula_response_type import ScalarFormulaResponseType

        return {
            "attributes": (ScalarFormulaResponseAtrributes,),
            "type": (ScalarFormulaResponseType,),
        }

    attribute_map = {
        "attributes": "attributes",
        "type": "type",
    }

    def __init__(
        self_,
        attributes: Union[ScalarFormulaResponseAtrributes, UnsetType] = unset,
        type: Union[ScalarFormulaResponseType, UnsetType] = unset,
        **kwargs,
    ):
        """
        A message containing the response to a scalar query.

        :param attributes: The object describing a scalar response.
        :type attributes: ScalarFormulaResponseAtrributes, optional

        :param type: The type of the resource. The value should always be scalar_response.
        :type type: ScalarFormulaResponseType, optional
        """
        if attributes is not unset:
            kwargs["attributes"] = attributes
        if type is not unset:
            kwargs["type"] = type
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class TargetFormatType(ModelSimple):
    """
    If the `target_type` of the remapper is `attribute`, try to cast the value to a new specific type.
        If the cast is not possible, the original type is kept. `string`, `integer`, or `double` are the possible types.
        If the `target_type` is `tag`, this parameter may not be specified.

    :param value: Must be one of ["auto", "string", "integer", "double"].
    :type value: str
    """

    allowed_values = {
        "auto",
        "string",
        "integer",
        "double",
    }
    AUTO: ClassVar["TargetFormatType"]
    STRING: ClassVar["TargetFormatType"]
    INTEGER: ClassVar["TargetFormatType"]
    DOUBLE: ClassVar["TargetFormatType"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


TargetFormatType.AUTO = TargetFormatType("auto")
TargetFormatType.STRING = TargetFormatType("string")
TargetFormatType.INTEGER = TargetFormatType("integer")
TargetFormatType.DOUBLE = TargetFormatType("double")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.usage_attribution_metadata import UsageAttributionMetadata
    from datadog_api_client.v1.model.usage_attribution_body import UsageAttributionBody


class UsageAttributionResponse(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.usage_attribution_metadata import UsageAttributionMetadata
        from datadog_api_client.v1.model.usage_attribution_body import UsageAttributionBody

        return {
            "metadata": (UsageAttributionMetadata,),
            "usage": ([UsageAttributionBody],),
        }

    attribute_map = {
        "metadata": "metadata",
        "usage": "usage",
    }

    def __init__(
        self_,
        metadata: Union[UsageAttributionMetadata, UnsetType] = unset,
        usage: Union[List[UsageAttributionBody], UnsetType] = unset,
        **kwargs,
    ):
        """
        Response containing the Usage Summary by tag(s).

        :param metadata: The object containing document metadata.
        :type metadata: UsageAttributionMetadata, optional

        :param usage: Get usage summary by tag(s).
        :type usage: [UsageAttributionBody], optional
        """
        if metadata is not unset:
            kwargs["metadata"] = metadata
        if usage is not unset:
            kwargs["usage"] = usage
        super().__init__(kwargs)



"""
Create a new dashboard with topology_map widget
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.topology_map_widget_definition import TopologyMapWidgetDefinition
from datadog_api_client.v1.model.topology_map_widget_definition_type import TopologyMapWidgetDefinitionType
from datadog_api_client.v1.model.topology_query import TopologyQuery
from datadog_api_client.v1.model.topology_query_data_source import TopologyQueryDataSource
from datadog_api_client.v1.model.topology_request import TopologyRequest
from datadog_api_client.v1.model.topology_request_type import TopologyRequestType
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_layout import WidgetLayout
from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign

body = Dashboard(
    title="Example-Create_a_new_dashboard_with_topology_map_widget",
    description="",
    widgets=[
        Widget(
            layout=WidgetLayout(
                x=0,
                y=0,
                width=47,
                height=15,
            ),
            definition=TopologyMapWidgetDefinition(
                title="",
                title_size="16",
                title_align=WidgetTextAlign.LEFT,
                type=TopologyMapWidgetDefinitionType.TOPOLOGY_MAP,
                requests=[
                    TopologyRequest(
                        request_type=TopologyRequestType.TOPOLOGY,
                        query=TopologyQuery(
                            data_source=TopologyQueryDataSource.SERVICE_MAP,
                            service="",
                            filters=[
                                "env:none",
                                "environment:*",
                            ],
                        ),
                    ),
                ],
            ),
        ),
    ],
    template_variables=[],
    layout_type=DashboardLayoutType.FREE,
    is_read_only=False,
    notify_list=[],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



"""
Create an SLO object returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.service_level_objectives_api import ServiceLevelObjectivesApi
from datadog_api_client.v1.model.service_level_objective_query import ServiceLevelObjectiveQuery
from datadog_api_client.v1.model.service_level_objective_request import ServiceLevelObjectiveRequest
from datadog_api_client.v1.model.slo_threshold import SLOThreshold
from datadog_api_client.v1.model.slo_timeframe import SLOTimeframe
from datadog_api_client.v1.model.slo_type import SLOType

body = ServiceLevelObjectiveRequest(
    type=SLOType.METRIC,
    description="string",
    groups=[
        "env:test",
        "role:mysql",
    ],
    monitor_ids=[],
    name="Example-Create_an_SLO_object_returns_OK_response",
    query=ServiceLevelObjectiveQuery(
        denominator="sum:httpservice.hits{!code:3xx}.as_count()",
        numerator="sum:httpservice.hits{code:2xx}.as_count()",
    ),
    tags=[
        "env:prod",
        "app:core",
    ],
    thresholds=[
        SLOThreshold(
            target=95.0,
            target_display="95.0",
            timeframe=SLOTimeframe.SEVEN_DAYS,
            warning=98.0,
            warning_display="98.0",
        ),
    ],
    timeframe=SLOTimeframe.SEVEN_DAYS,
    target_threshold=97.0,
    warning_threshold=98.0,
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = ServiceLevelObjectivesApi(api_client)
    response = api_instance.create_slo(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class SecurityMonitoringRuleMaxSignalDuration(ModelSimple):
    """
    A signal will “close” regardless of the query being matched once the time exceeds the maximum duration.
        This time is calculated from the first seen timestamp.

    :param value: Must be one of [0, 60, 300, 600, 900, 1800, 3600, 7200, 10800, 21600, 43200, 86400].
    :type value: int
    """

    allowed_values = {
        0,
        60,
        300,
        600,
        900,
        1800,
        3600,
        7200,
        10800,
        21600,
        43200,
        86400,
    }
    ZERO_MINUTES: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    ONE_MINUTE: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    FIVE_MINUTES: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    TEN_MINUTES: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    FIFTEEN_MINUTES: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    THIRTY_MINUTES: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    ONE_HOUR: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    TWO_HOURS: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    THREE_HOURS: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    SIX_HOURS: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    TWELVE_HOURS: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]
    ONE_DAY: ClassVar["SecurityMonitoringRuleMaxSignalDuration"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (int,),
        }


SecurityMonitoringRuleMaxSignalDuration.ZERO_MINUTES = SecurityMonitoringRuleMaxSignalDuration(0)
SecurityMonitoringRuleMaxSignalDuration.ONE_MINUTE = SecurityMonitoringRuleMaxSignalDuration(60)
SecurityMonitoringRuleMaxSignalDuration.FIVE_MINUTES = SecurityMonitoringRuleMaxSignalDuration(300)
SecurityMonitoringRuleMaxSignalDuration.TEN_MINUTES = SecurityMonitoringRuleMaxSignalDuration(600)
SecurityMonitoringRuleMaxSignalDuration.FIFTEEN_MINUTES = SecurityMonitoringRuleMaxSignalDuration(900)
SecurityMonitoringRuleMaxSignalDuration.THIRTY_MINUTES = SecurityMonitoringRuleMaxSignalDuration(1800)
SecurityMonitoringRuleMaxSignalDuration.ONE_HOUR = SecurityMonitoringRuleMaxSignalDuration(3600)
SecurityMonitoringRuleMaxSignalDuration.TWO_HOURS = SecurityMonitoringRuleMaxSignalDuration(7200)
SecurityMonitoringRuleMaxSignalDuration.THREE_HOURS = SecurityMonitoringRuleMaxSignalDuration(10800)
SecurityMonitoringRuleMaxSignalDuration.SIX_HOURS = SecurityMonitoringRuleMaxSignalDuration(21600)
SecurityMonitoringRuleMaxSignalDuration.TWELVE_HOURS = SecurityMonitoringRuleMaxSignalDuration(43200)
SecurityMonitoringRuleMaxSignalDuration.ONE_DAY = SecurityMonitoringRuleMaxSignalDuration(86400)



import hashlib
import json
import pathlib
import random
import uuid
import yaml

from jsonref import JsonRef
from yaml import CSafeLoader

from . import formatter

PRIMITIVE_TYPES = ["string", "number", "boolean", "integer"]


def load(filename):
    path = pathlib.Path(filename)
    with path.open() as fp:
        return JsonRef.replace_refs(yaml.load(fp, Loader=CSafeLoader))


def type_to_python_helper(type_, schema, alternative_name=None, in_list=False):
    if type_ is None:
        return "bool, date, datetime, dict, float, int, list, str, none_type"

    if type_ == "integer":
        return "int"
    elif type_ == "number":
        return "float"
    elif type_ == "string":
        format_ = schema.get("format")
        if format_ in {"date", "date-time"}:
            return "datetime"
        elif format_ == "binary":
            return "file_type"
        return "str"
    elif type_ == "boolean":
        return "bool"
    elif type_ == "array":
        subtype = type_to_python(schema["items"], in_list=True)
        if schema["items"].get("nullable"):
             subtype += ", none_type"
        return "[{}]".format(subtype)
    elif type_ == "object":
        if "additionalProperties" in schema:
            nested_schema = schema["additionalProperties"]
            nested_name = type_to_python(nested_schema)
            if nested_schema.get("nullable"):
                nested_name += ", none_type"
            return "{{str: ({},)}}".format(nested_name)
        return (
            alternative_name
            if alternative_name
            and ("properties" in schema or "oneOf" in schema or "anyOf" in schema or "allOf" in schema)
            else "dict"
        )
    elif type_ == "null":
        return "none_type"
    else:
        raise ValueError(f"Unknown type {type_}")


def type_to_python(schema, alternative_name=None, in_list=False):
    """Return Python type name for the type."""
    name = formatter.get_name(schema)
    if name:
        if "enum" in schema:
            return name
        if schema.get("type", "object") in ("object", "array"):
            return name

    type_ = schema.get("type")
    if type_ is None:
        if "oneOf" in schema and in_list:
            type_ = ""
            for child in schema["oneOf"]:
                # We do not generate model for nested primitive oneOfs
                if in_list and "items" in child and child["items"].get("type") in PRIMITIVE_TYPES:
                    type_ += f"{type_to_python_helper(child.get('type'), child, in_list=in_list)},"
                else:
                    type_ += f"{type_to_python(child, in_list=in_list)},"
            return type_
        if "items" in schema:
            type_ = "array"

    return type_to_python_helper(type_, schema, alternative_name=alternative_name, in_list=in_list)


def get_type_for_attribute(schema, attribute, current_name=None):
    """Return Python type name for the attribute."""
    child_schema = schema.get("properties", {}).get(attribute)
    alternative_name = current_name + formatter.camel_case(attribute) if current_name else None
    return type_to_python(child_schema, alternative_name=alternative_name)


def typing_to_python_helper(type_, schema, alternative_name=None, in_list=False):
    if type_ is None:
        return "Any"

    if type_ == "integer":
        return "int"
    elif type_ == "number":
        return "float"
    elif type_ == "string":
        format_ = schema.get("format")
        if format_ in {"date", "date-time"}:
            return "datetime"
        elif format_ == "binary":
            return "file_type"
        return "str"
    elif type_ == "boolean":
        return "bool"
    elif type_ == "array":
        return "List[{}]".format(typing_to_python(schema["items"], in_list=True))
    elif type_ == "object":
        if "additionalProperties" in schema:
            nested_schema = schema["additionalProperties"]
            nested_name = typing_to_python(nested_schema)
            if nested_schema.get("nullable"):
                nested_name = f"Union[{nested_name}, none_type]"
            return f"Dict[str, {nested_name}]"
        return (
            alternative_name
            if alternative_name
            and ("properties" in schema or "oneOf" in schema or "anyOf" in schema or "allOf" in schema)
            else "dict"
        )
    elif type_ == "null":
        return "none_type"
    else:
        raise ValueError(f"Unknown type {type_}")


def typing_to_python(schema, alternative_name=None, in_list=False):
    """Return Python type name for the type."""
    name = formatter.get_name(schema)
    if name:
        if "enum" in schema:
            return name
        if schema.get("type", "object") in ("object", "array"):
            if "oneOf" in schema:
                types = [name]
                types.extend(get_oneof_types(schema))
                return f"Union[{','.join(types)}]"
            return name

    type_ = schema.get("type")
    if type_ is None:
        if "oneOf" in schema and in_list:
            type_ = ""
            for child in schema["oneOf"]:
                # We do not generate model for nested primitive oneOfs
                if in_list and "items" in child and child["items"].get("type") in PRIMITIVE_TYPES:
                    type_ += f"{typing_to_python_helper(child.get('type'), child, in_list=in_list)},"
                else:
                    type_ += f"{typing_to_python(child, in_list=in_list)},"
            return type_
        if "items" in schema:
            type_ = "array"

    return typing_to_python_helper(type_, schema, alternative_name=alternative_name, in_list=in_list)


def get_typing_for_attribute(schema, attribute, current_name=None, optional=False):
    child_schema = schema.get("properties", {}).get(attribute)
    alternative_name = current_name + formatter.camel_case(attribute) if current_name else None
    attr_type = typing_to_python(child_schema, alternative_name=alternative_name)
    if child_schema.get("nullable"):
        if optional:
            return f"Union[{attr_type}, none_type, UnsetType]"
        return f"Union[{attr_type}, none_type]"
    if optional:
        if attr_type.startswith("Union"):
            return attr_type[:-1] + ", UnsetType]"
        return f"Union[{attr_type}, UnsetType]"
    return attr_type


def get_types_for_attribute(schema, attribute, current_name=None):
    child_schema = schema.get("properties", {}).get(attribute)
    base_type = get_type_for_attribute(schema, attribute, current_name)
    if child_schema.get("nullable") and not formatter.get_name(child_schema):
        return f"({base_type}, none_type)"
    return f"({base_type},)"


def get_type_for_items(schema):
    name = formatter.get_name(schema.get("items"))
    return name


def get_type_for_parameter(parameter, typing=False):
    """Return Python type name for the parameter."""
    if "content" in parameter:
        assert "in" not in parameter
        for content in parameter["content"].values():
            data = type_to_python(content["schema"])
            if typing:
                if "oneOf" in content["schema"]:
                    types = [data]
                    types.extend(get_oneof_types(content["schema"]))
                    return f"Union[{','.join(types)}]"
                data = data.replace("[", "List[")
            return data
    data = type_to_python(parameter.get("schema"))
    if typing:
        data = data.replace("[", "List[")
    return data


def get_enum_type(schema):
    type_ = schema.get("type")

    if type_ == "integer":
        return "int"
    elif type_ == "string":
        return "str"


def get_enum_default(model):
    return model["enum"][0] if len(model["enum"]) == 1 else model.get("default")


def child_models(schema, alternative_name=None, seen=None, in_list=False):
    seen = seen or set()
    current_name = formatter.get_name(schema)
    name = current_name or alternative_name

    has_sub_models = False
    if "allOf" in schema:
        has_sub_models = True
        for child in schema["allOf"]:
            yield from child_models(child, seen=seen)
    if "oneOf" in schema:
        has_sub_models = True
        for child in schema["oneOf"]:
            # Don't generate models for nested primitive types
            if in_list and child.get("type") in PRIMITIVE_TYPES:
                return
            yield from child_models(child, seen=seen)
    if "anyOf" in schema:
        has_sub_models = True
        for child in schema["anyOf"]:
            yield from child_models(child, seen=seen)

    if "items" in schema:
        yield from child_models(schema["items"], None, seen=seen, in_list=True)

    if schema.get("type") == "object" or "properties" in schema or has_sub_models:
        if not has_sub_models and name is None:
            # this is a basic map object so we don't need a type
            return

        if name is None:
            return

        if name in seen:
            return

        if "properties" in schema or has_sub_models:
            seen.add(name)
            yield name, schema

        if "additionalProperties" in schema and current_name:
            seen.add(name)
            yield name, schema

        for key, child in schema.get("properties", {}).items():
            yield from child_models(child, alternative_name=name + formatter.camel_case(key), seen=seen)

    if current_name and schema.get("type") == "array":
        if name in seen:
            return

        seen.add(name)
        yield name, schema

    if "enum" in schema:
        if name is None:
            raise ValueError(f"Schema {schema} has no name")

        if name in seen:
            return

        seen.add(name)
        yield name, schema

    if "additionalProperties" in schema:
        nested_name = formatter.get_name(schema["additionalProperties"])
        if nested_name:
            yield from child_models(
                schema["additionalProperties"],
                alternative_name=name,
                seen=seen,
            )


def models(spec):
    name_to_schema = {}

    for path in spec["paths"]:
        for method in spec["paths"][path]:
            operation = spec["paths"][path][method]

            for content in operation.get("parameters", []):
                if "schema" in content:
                    name_to_schema.update(dict(child_models(content["schema"])))

            for content in operation.get("requestBody", {}).get("content", {}).values():
                if "schema" in content:
                    name_to_schema.update(dict(child_models(content["schema"])))

            for response in operation.get("responses", {}).values():
                for content in response.get("content", {}).values():
                    if "schema" in content:
                        name_to_schema.update(dict(child_models(content["schema"])))

    return name_to_schema


def get_references_for_model(model, model_name):
    result = {}
    top_name = formatter.get_name(model) or model_name
    for key, definition in model.get("properties", {}).items():
        if definition.get("type") == "object" or definition.get("enum") or definition.get("oneOf"):
            name = formatter.get_name(definition)
            if name:
                result[name] = None
            elif definition.get("properties") and top_name:
                result[top_name + formatter.camel_case(key)] = None
            elif definition.get("additionalProperties"):
                name = formatter.get_name(definition["additionalProperties"])
                if name:
                    result[name] = None
        elif definition.get("type") == "array":
            name = formatter.get_name(definition)
            if name:
                result[name] = None
            else:
                name = formatter.get_name(definition.get("items"))
                if name:
                    result[name] = None
        elif definition.get("properties") and top_name:
            result[top_name + formatter.camel_case(key)] = None
    if model.get("additionalProperties"):
        definition = model["additionalProperties"]
        name = formatter.get_name(definition)
        if name:
            result[name] = None
        elif definition.get("type") == "array":
            name = formatter.get_name(definition.get("items"))
            if name:
                result[name] = None
    result.pop(model_name, None)
    return list(result)


def get_oneof_references_for_model(model, model_name, seen=None):
    result = {}
    if seen is None:
        seen = set()
    name = formatter.get_name(model)
    if name:
        if name in seen:
            return []
        seen.add(name)

    if model.get("oneOf"):
        for schema in model["oneOf"]:
            type_ = schema.get("type", "object")
            if type_ in ("array", "object"):
                result[formatter.get_name(schema)] = None

    for key, definition in model.get("properties", {}).items():
        result.update({k: None for k in get_oneof_references_for_model(definition, model_name, seen)})
        if definition.get("items"):
            result.update({k: None for k in get_oneof_references_for_model(definition["items"], model_name, seen)})
        if definition.get("additionalProperties"):
            result.update(
                {k: None for k in get_oneof_references_for_model(definition["additionalProperties"], model_name, seen)}
            )
    result.pop(model_name, None)
    return list(result)


def get_oneof_parameters(model):
    seen = set()
    for schema in model["oneOf"]:
        for attr, definition in schema.get("properties", {}).items():
            if attr not in seen:
                seen.add(attr)
                yield attr, definition, schema


def get_oneof_types(model):
    for schema in model["oneOf"]:
        type_ = schema.get("type", "object")
        if type_ in ("array", "object"):
            yield formatter.get_name(schema)
        elif type_ == "integer":
            yield "int"
        elif type_ == "string":
            yield "str"
        elif type_ == "number":
            yield "float"
        elif type_ == "boolean":
            yield "bool"
        else:
            raise NotImplementedError(f"Type {type_} not implemented")


def get_oneof_models(model):
    result = []
    for schema in model["oneOf"]:
        if schema.get("type", "object") in ("array", "object"):
            result.append(formatter.get_name(schema))
    return result


def apis(spec):
    operations = {}

    for path in spec["paths"]:
        for method in spec["paths"][path]:
            operation = spec["paths"][path][method]
            tag = operation.get("tags", [None])[0]
            operations.setdefault(tag, []).append((path, method, operation))

    return operations


def get_api_models(operations):
    seen = set()
    for _, _, operation in operations:
        for response in operation.get("responses", {}).values():
            for content in response.get("content", {}).values():
                if "schema" in content:
                    name = formatter.get_name(content["schema"])
                    if name and name not in seen:
                        seen.add(name)
                        yield name
                    elif "items" in content["schema"]:
                        name = formatter.get_name(content["schema"]["items"])
                        if name and name not in seen:
                            seen.add(name)
                            yield name
            break
        for content in operation.get("parameters", []):
            if "schema" in content and (
                content["schema"].get("type") in ("object", "array") or content["schema"].get("enum")
            ):
                name = formatter.get_name(content["schema"])
                if name and name not in seen:
                    seen.add(name)
                    yield name
                elif "items" in content["schema"]:
                    name = formatter.get_name(content["schema"]["items"])
                    if name and name not in seen:
                        seen.add(name)
                        yield name
        if "requestBody" in operation:
            for content in operation["requestBody"].get("content", {}).values():
                if "schema" in content:
                    name = formatter.get_name(content["schema"])
                    if name and name not in seen:
                        seen.add(name)
                        yield name
                        if "oneOf" in content["schema"]:
                            for schema in content["schema"]["oneOf"]:
                                type_ = schema.get("type", "object")
                                if type_ in ("array", "object"):
                                    name = formatter.get_name(schema)
                                    if name and name not in seen:
                                        seen.add(name)
                                        yield name
        if "x-pagination" in operation:
            name = get_type_at_path(operation, operation["x-pagination"]["resultsPath"])
            if name and name not in seen:
                seen.add(name)
                yield name


def parameters(operation):
    for content in operation.get("parameters", []):
        if "schema" in content:
            yield content["name"], content

    if "requestBody" in operation:
        if "multipart/form-data" in operation["requestBody"]["content"]:
            parent = operation["requestBody"]["content"]["multipart/form-data"]["schema"]
            for name, schema in parent["properties"].items():
                yield name, {
                    "in": "form",
                    "schema": schema,
                    "name": name,
                    "description": schema.get("description"),
                    "required": name in parent.get("required", []),
                }
        else:
            name = operation.get("x-codegen-request-body-name", "body")
            yield name, operation["requestBody"]


def parameter_schema(parameter):
    if "schema" in parameter:
        return parameter["schema"]
    if "content" in parameter:
        for content in parameter.get("content", {}).values():
            if "schema" in content:
                return content["schema"]
    raise ValueError(f"Unknown schema for parameter {parameter}")


def return_type(operation):
    for response in operation.get("responses", {}).values():
        for content in response.get("content", {}).values():
            if "schema" in content:
                return type_to_python(content["schema"])
        return


def accept_headers(operation):
    any_type = "*/*"
    seen = []
    for response in operation.get("responses", {}).values():
        if "content" in response:
            for media_type in response["content"].keys():
                if media_type not in seen:
                    seen.append(media_type)
        else:
            return [any_type]
    return seen


def collection_format(parameter):
    in_to_style = {
        "query": "form",
        "path": "simple",
        "header": "simple",
        "cookie": "form",
    }
    schema = parameter_schema(parameter)
    matrix = {
        ("form", False): "csv",
        ("form", True): "multi",
        # TODO add more cases from https://swagger.io/specification/#parameter-style
    }
    if schema.get("type") == "array" or "items" in schema:
        in_ = parameter.get("in", "query")
        style = parameter.get("style", in_to_style[in_])
        explode = parameter.get("explode", True if style == "form" else False)
        return matrix.get((style, explode), "multi")


def generate_value(schema, use_random=False, prefix=None):
    spec = schema.spec
    if not use_random:
        if "example" in spec:
            return spec["example"]
        if "default" in spec:
            return spec["default"]

    if spec["type"] == "string":
        if use_random:
            return str(
                uuid.UUID(
                    bytes=hashlib.sha256(
                        str(prefix or schema.keys).encode("utf-8"),
                    ).digest()[:16]
                )
            )
        return "string"
    elif spec["type"] == "integer":
        return random.randint(0, 32000) if use_random else len(str(prefix or schema.keys))
    elif spec["type"] == "number":
        return random.random() if use_random else 1.0 / len(str(prefix or schema.keys))
    elif spec["type"] == "boolean":
        return True
    elif spec["type"] == "array":
        return [generate_value(schema[0], use_random=use_random)]
    elif spec["type"] == "object":
        return {key: generate_value(schema[key], use_random=use_random) for key in spec["properties"]}
    else:
        raise TypeError(f"Unknown type: {spec['type']}")


class Schema:
    def __init__(self, spec, value=None, keys=None):
        self.spec = spec
        self.value = value if value is not None else generate_value
        self.keys = keys or tuple()

    def __getattr__(self, key):
        return self[key]

    def __getitem__(self, key):
        type_ = self.spec.get("type", "object")
        if type_ == "object":
            try:
                return self.__class__(
                    self.spec["properties"][key],
                    value=self.value,
                    keys=self.keys + (key,),
                )
            except KeyError:
                if "oneOf" in self.spec:
                    for schema in self.spec["oneOf"]:
                        if schema.get("type", "object") == "object":
                            try:
                                return self.__class__(
                                    schema["properties"][key],
                                    value=self.value,
                                    keys=self.keys + (key,),
                                )
                            except KeyError:
                                pass
            raise KeyError(f"{key} not found in {self.spec.get('properties', {}).keys()}: {self.spec}")
        if type_ == "array":
            return self.__class__(self.spec["items"], value=self.value, keys=self.keys + (key,))

        raise KeyError(f"{key} not found in {self.spec}")

    def __repr__(self):
        value = self.value(self)
        if isinstance(value, (dict, list)):
            return json.dumps(value, indent=2)
        return str(value)


class Operation:
    def __init__(self, name, spec, method, path):
        self.name = name
        self.spec = spec
        self.method = method
        self.path = path

    def server_url_and_method(self, spec, server_index=0, server_variables=None):
        def format_server(server, path):
            url = server["url"] + path
            # replace potential path variables
            for variable, value in server_variables.items():
                url = url.replace("{" + variable + "}", value)
            # replace server variables if they were not replace before
            for variable in server["variables"]:
                if variable in server_variables:
                    continue
                url = url.replace("{" + variable + "}", server["variables"][variable]["default"])
            return url

        server_variables = server_variables or {}
        if "servers" in self.spec:
            server = self.spec["servers"][server_index]
        else:
            server = spec["servers"][server_index]
        return format_server(server, self.path), self.method

    def response_code_and_accept_type(self):
        for response in self.spec["responses"]:
            return int(response), next(iter(self.spec["responses"][response].get("content", {None: None})))
        return None, None

    def request_content_type(self):
        return next(iter(self.spec.get("requestBody", {}).get("content", {None: None})))

    def response(self):
        for response in self.spec["responses"]:
            return Schema(next(iter((self.spec["responses"][response]["content"].values())))["schema"])

    def request(self):
        return Schema(next(iter(self.spec["requestBody"]["content"].values()))["schema"])


def get_default(operation, attribute_path):
    attrs = attribute_path.split(".")
    for name, parameter in parameters(operation):
        if name == attrs[0]:
            break
    if name == attribute_path:
        # We found a top level attribute matching the full path, let's use the default
        return parameter["schema"]["default"]

    if name == "body":
        parameter = next(iter(parameter["content"].values()))["schema"]
    for attr in attrs[1:]:
        parameter = parameter["properties"][attr]
    return parameter["default"]


def get_type_at_path(operation, attribute_path):
    content = None
    for code, response in operation.get("responses", {}).items():
        if int(code) >= 300:
            continue
        for content in response.get("content", {}).values():
            if "schema" in content:
                break
    if content is None:
        raise RuntimeError("Default response not found")
    for attr in attribute_path.split("."):
        content = content["schema"]["properties"][attr]
    return get_type_for_items(content)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class ProcessSummaryAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "cmdline": (str,),
            "host": (str,),
            "pid": (int,),
            "ppid": (int,),
            "start": (str,),
            "tags": ([str],),
            "timestamp": (str,),
            "user": (str,),
        }

    attribute_map = {
        "cmdline": "cmdline",
        "host": "host",
        "pid": "pid",
        "ppid": "ppid",
        "start": "start",
        "tags": "tags",
        "timestamp": "timestamp",
        "user": "user",
    }

    def __init__(
        self_,
        cmdline: Union[str, UnsetType] = unset,
        host: Union[str, UnsetType] = unset,
        pid: Union[int, UnsetType] = unset,
        ppid: Union[int, UnsetType] = unset,
        start: Union[str, UnsetType] = unset,
        tags: Union[List[str], UnsetType] = unset,
        timestamp: Union[str, UnsetType] = unset,
        user: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Attributes for a process summary.

        :param cmdline: Process command line.
        :type cmdline: str, optional

        :param host: Host running the process.
        :type host: str, optional

        :param pid: Process ID.
        :type pid: int, optional

        :param ppid: Parent process ID.
        :type ppid: int, optional

        :param start: Time the process was started.
        :type start: str, optional

        :param tags: List of tags associated with the process.
        :type tags: [str], optional

        :param timestamp: Time the process was seen.
        :type timestamp: str, optional

        :param user: Process owner.
        :type user: str, optional
        """
        if cmdline is not unset:
            kwargs["cmdline"] = cmdline
        if host is not unset:
            kwargs["host"] = host
        if pid is not unset:
            kwargs["pid"] = pid
        if ppid is not unset:
            kwargs["ppid"] = ppid
        if start is not unset:
            kwargs["start"] = start
        if tags is not unset:
            kwargs["tags"] = tags
        if timestamp is not unset:
            kwargs["timestamp"] = timestamp
        if user is not unset:
            kwargs["user"] = user
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelComposed,
    cached_property,
)


class ScalarQuery(ModelComposed):
    def __init__(self, **kwargs):
        """
        An individual scalar query to one of the basic Datadog data sources.

        :param aggregator: The type of aggregation that can be performed on metrics queries.
        :type aggregator: MetricsAggregator

        :param data_source: A data source that is powered by the Metrics backend.
        :type data_source: MetricsDataSource

        :param name: The variable name for use in formulas.
        :type name: str, optional

        :param query: A classic metrics query string.
        :type query: str

        :param compute: The instructions for what to compute for this query.
        :type compute: EventsCompute

        :param group_by: The list of facets on which to split results.
        :type group_by: EventsQueryGroupBys, optional

        :param indexes: The indexes in which to search.
        :type indexes: [str], optional

        :param search: Configuration of the search/filter for an events query.
        :type search: EventsSearch, optional
        """
        super().__init__(kwargs)

    @cached_property
    def _composed_schemas(_):
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error because the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        from datadog_api_client.v2.model.metrics_scalar_query import MetricsScalarQuery
        from datadog_api_client.v2.model.events_scalar_query import EventsScalarQuery

        return {
            "oneOf": [
                MetricsScalarQuery,
                EventsScalarQuery,
            ],
        }



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.incident_attachments_postmortem_attributes_attachment_object import (
        IncidentAttachmentsPostmortemAttributesAttachmentObject,
    )
    from datadog_api_client.v2.model.incident_attachment_postmortem_attachment_type import (
        IncidentAttachmentPostmortemAttachmentType,
    )


class IncidentAttachmentPostmortemAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.incident_attachments_postmortem_attributes_attachment_object import (
            IncidentAttachmentsPostmortemAttributesAttachmentObject,
        )
        from datadog_api_client.v2.model.incident_attachment_postmortem_attachment_type import (
            IncidentAttachmentPostmortemAttachmentType,
        )

        return {
            "attachment": (IncidentAttachmentsPostmortemAttributesAttachmentObject,),
            "attachment_type": (IncidentAttachmentPostmortemAttachmentType,),
        }

    attribute_map = {
        "attachment": "attachment",
        "attachment_type": "attachment_type",
    }

    def __init__(
        self_,
        attachment: IncidentAttachmentsPostmortemAttributesAttachmentObject,
        attachment_type: IncidentAttachmentPostmortemAttachmentType,
        **kwargs,
    ):
        """
        The attributes object for a postmortem attachment.

        :param attachment: The postmortem attachment.
        :type attachment: IncidentAttachmentsPostmortemAttributesAttachmentObject

        :param attachment_type: The type of postmortem attachment attributes.
        :type attachment_type: IncidentAttachmentPostmortemAttachmentType
        """
        super().__init__(kwargs)

        self_.attachment = attachment
        self_.attachment_type = attachment_type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


class PermissionAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "created": (datetime,),
            "description": (str,),
            "display_name": (str,),
            "display_type": (str,),
            "group_name": (str,),
            "name": (str,),
            "restricted": (bool,),
        }

    attribute_map = {
        "created": "created",
        "description": "description",
        "display_name": "display_name",
        "display_type": "display_type",
        "group_name": "group_name",
        "name": "name",
        "restricted": "restricted",
    }

    def __init__(
        self_,
        created: Union[datetime, UnsetType] = unset,
        description: Union[str, UnsetType] = unset,
        display_name: Union[str, UnsetType] = unset,
        display_type: Union[str, UnsetType] = unset,
        group_name: Union[str, UnsetType] = unset,
        name: Union[str, UnsetType] = unset,
        restricted: Union[bool, UnsetType] = unset,
        **kwargs,
    ):
        """
        Attributes of a permission.

        :param created: Creation time of the permission.
        :type created: datetime, optional

        :param description: Description of the permission.
        :type description: str, optional

        :param display_name: Displayed name for the permission.
        :type display_name: str, optional

        :param display_type: Display type.
        :type display_type: str, optional

        :param group_name: Name of the permission group.
        :type group_name: str, optional

        :param name: Name of the permission.
        :type name: str, optional

        :param restricted: Whether or not the permission is restricted.
        :type restricted: bool, optional
        """
        if created is not unset:
            kwargs["created"] = created
        if description is not unset:
            kwargs["description"] = description
        if display_name is not unset:
            kwargs["display_name"] = display_name
        if display_type is not unset:
            kwargs["display_type"] = display_type
        if group_name is not unset:
            kwargs["group_name"] = group_name
        if name is not unset:
            kwargs["name"] = name
        if restricted is not unset:
            kwargs["restricted"] = restricted
        super().__init__(kwargs)



"""
Get all app keys owned by this service account returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.service_accounts_api import ServiceAccountsApi

# there is a valid "service_account_user" in the system
SERVICE_ACCOUNT_USER_DATA_ID = environ["SERVICE_ACCOUNT_USER_DATA_ID"]

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = ServiceAccountsApi(api_client)
    response = api_instance.list_service_account_application_keys(
        service_account_id=SERVICE_ACCOUNT_USER_DATA_ID,
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class LogsMetricUpdateCompute(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "include_percentiles": (bool,),
        }

    attribute_map = {
        "include_percentiles": "include_percentiles",
    }

    def __init__(self_, include_percentiles: Union[bool, UnsetType] = unset, **kwargs):
        """
        The compute rule to compute the log-based metric.

        :param include_percentiles: Toggle to include or exclude percentile aggregations for distribution metrics.
            Only present when the ``aggregation_type`` is ``distribution``.
        :type include_percentiles: bool, optional
        """
        if include_percentiles is not unset:
            kwargs["include_percentiles"] = include_percentiles
        super().__init__(kwargs)



"""
Get all processes returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.processes_api import ProcessesApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = ProcessesApi(api_client)
    response = api_instance.list_processes(
        search="process-agent",
        tags="testing:true",
        page_limit=2,
    )

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.

from contextlib import suppress
from datetime import date, datetime
import enum
import inspect
import io
import os
import pprint
import re
import tempfile
from types import MappingProxyType
from typing import Collection, Mapping, Union
from typing_extensions import Final

from dateutil.parser import parse

from datadog_api_client.exceptions import (
    ApiKeyError,
    ApiAttributeError,
    ApiTypeError,
    ApiValueError,
)

none_type = type(None)
file_type = io.IOBase
empty_dict = MappingProxyType({})  # type: ignore


class UnsetType(enum.Enum):
    unset = 0


unset: Final = UnsetType.unset


class cached_property(object):
    # This caches the result of the function call for fn with no inputs
    # use this as a decorator on function methods that you want converted
    # into cached properties
    result_key = "_results"

    def __init__(self, fn):
        self._fn = fn

    def __get__(self, instance, cls=None):
        if self.result_key in vars(self):
            return vars(self)[self.result_key]
        else:
            result = self._fn(instance)
            setattr(self, self.result_key, result)
            return result


PRIMITIVE_TYPES = (list, float, int, bool, datetime, date, str, file_type)


def allows_single_value_input(cls):
    """
    This function returns True if the input composed schema model or any
    descendant model allows a value only input.
    """
    if issubclass(cls, ModelSimple) or cls in PRIMITIVE_TYPES:
        return True
    elif issubclass(cls, ModelComposed):
        if not cls._composed_schemas["oneOf"]:
            return False
        return any(allows_single_value_input(c) for c in cls._composed_schemas["oneOf"])
    return False


def composed_model_input_classes(cls):
    """
    This function returns a list of the possible models that can be accepted as
    inputs.
    """
    if issubclass(cls, ModelSimple) or cls in PRIMITIVE_TYPES:
        return [cls]
    elif issubclass(cls, ModelNormal):
        return [cls]
    elif issubclass(cls, ModelComposed):
        if not cls._composed_schemas["oneOf"]:
            return []
        input_classes = []
        for c in cls._composed_schemas["oneOf"]:
            input_classes.extend(composed_model_input_classes(c))
        return input_classes
    return []


class OpenApiModel(object):
    """The base class for all OpenAPIModels.

    :var attribute_map: The key is attribute name and the value is json
        key in definition.
    :type attribute_map: dict
    :var validations: The key is the name of the attribute. The value is a dict
        that stores validations for max_length, min_length, max_items,
        min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
        inclusive_minimum, and regex.
    :type validations: dict
    :var additional_properties_type: A tuple of classes accepted
        as additional properties values.
    :type additional_properties_type: tuple
    """

    _composed_schemas = empty_dict

    additional_properties_type = None

    attribute_map: Mapping[str, str] = empty_dict

    _nullable = False

    validations: Mapping[str, Mapping[str, int]] = empty_dict

    openapi_types = empty_dict

    read_only_vars: Collection[str] = frozenset()

    def set_attribute(self, name, value):
        # this is only used to set properties on self

        path_to_item = []
        if self._path_to_item:
            path_to_item.extend(self._path_to_item)
        path_to_item.append(name)

        if name in self.openapi_types:
            required_types_mixed = self.openapi_types[name]
        elif self.additional_properties_type is None:
            raise ApiAttributeError("{0} has no attribute '{1}'".format(type(self).__name__, name), path_to_item)
        elif self.additional_properties_type is not None:
            required_types_mixed = self.additional_properties_type

        if get_simple_class(name) != str:
            error_msg = type_error_message(var_name=name, var_value=name, valid_classes=(str,), key_type=True)
            raise ApiTypeError(error_msg, path_to_item=path_to_item, valid_classes=(str,), key_type=True)

        if self._check_type and value is not None:
            value = validate_and_convert_types(
                value,
                required_types_mixed,
                path_to_item,
                self._spec_property_naming,
                self._check_type,
                configuration=self._configuration,
            )
        if name in self.validations:
            check_validations(self.validations[name], name, value, self._configuration)
        self.__dict__["_data_store"][name] = value
        if isinstance(value, OpenApiModel) and value._unparsed:
            self._unparsed = True

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other

    def __setattr__(self, attr, value):
        """Set the value of an attribute using dot notation: `instance.attr = val`."""
        self[attr] = value

    def __getattr__(self, attr):
        """Get the value of an attribute using dot notation: `instance.attr`."""
        return self.__getitem__(attr)

    def __new__(cls, *args, **kwargs):
        if len(args) == 1:
            arg = args[0]
            if arg is None and is_type_nullable(cls):
                # The input data is the 'null' value and the type is nullable.
                return None

            if issubclass(cls, ModelComposed) and allows_single_value_input(cls):
                model_kwargs = {}
                oneof_instance = get_oneof_instance(cls, model_kwargs, kwargs, model_arg=arg)
                return oneof_instance

        return super(OpenApiModel, cls).__new__(cls)

    @classmethod
    def _new_from_openapi_data(cls_, *args, **kwargs):
        kwargs = change_keys_js_to_python(kwargs, cls_)
        if len(args) == 1:
            arg = args[0]
            if arg is None and is_type_nullable(cls_):
                # The input data is the 'null' value and the type is nullable.
                return None

            if issubclass(cls_, ModelComposed) and allows_single_value_input(cls_):
                model_kwargs = {}
                oneof_instance = get_oneof_instance(cls_, model_kwargs, kwargs, model_arg=arg)
                return oneof_instance

        self = super(OpenApiModel, cls_).__new__(cls_)
        cls_.__init__(self, *args, **kwargs)
        return self

    def __init__(self, kwargs):
        """
        :param _check_type: If True, values for parameters in openapi_types
            will be type checked and a TypeError will be raised if the wrong type is input.
            Defaults to True.
        :type _check_type: bool
        :param _path_to_item: This is a list of keys or values to drill down to
            the model in received_data when deserializing a response.
        :type _path_to_item: tuple/list
        :param _spec_property_naming: True if the variable names in the input
            data are serialized names, as specified in the OpenAPI document.  False if the
            variable names in the input data are pythonic names, e.g. snake case (default).
        :type _spec_property_naming: bool
        :param _configuration: The instance to use when deserializing a
            file_type parameter.  If passed, type conversion is attempted If omitted no
            type conversion is done.
        :type _configuration: Configuration
        """
        _check_type = kwargs.pop("_check_type", True)
        _spec_property_naming = kwargs.pop("_spec_property_naming", False)
        _path_to_item = kwargs.pop("_path_to_item", ())
        _configuration = kwargs.pop("_configuration", None)

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._unparsed = False

    def _check_pos_args(self, args):
        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments."
                % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=self._path_to_item,
                valid_classes=(self.__class__,),
            )

    def _check_kw_args(self, kwargs):
        if kwargs:
            raise ApiTypeError(
                "Invalid named arguments=%s passed to %s. Remove those invalid named arguments."
                % (
                    kwargs,
                    self.__class__.__name__,
                ),
                path_to_item=self._path_to_item,
                valid_classes=(self.__class__,),
            )


class ModelSimple(OpenApiModel):
    """
    The parent class of models whose type != object in their
    swagger/openapi.

    :var allowed_values: Set of allowed values.
    :type allowed_values: set
    """

    allowed_values: Collection[Union[str, int]] = frozenset()

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_spec_property_naming",
            "_path_to_item",
            "_configuration",
            "_unparsed",
        ]
    )

    def __init__(self, value, **kwargs):
        super().__init__(kwargs)
        self.value = value
        self._check_kw_args(kwargs)

    def __setitem__(self, name, value):
        """Set the value of an attribute using square-bracket notation: `instance[attr] = val`."""
        if name in self.required_properties:
            self.__dict__[name] = value
            return

        if self.allowed_values and name == "value":
            try:
                check_allowed_values(self.allowed_values, name, value)
            except ApiValueError:
                self.__dict__["_data_store"][name] = value
                self._unparsed = True
                return

        self.set_attribute(name, value)

    def get(self, name, default=None):
        """Returns the value of an attribute or some default value if the attribute was not set."""
        if name in self.required_properties:
            return self.__dict__[name]

        return self.__dict__["_data_store"].get(name, default)

    def __getitem__(self, name):
        """Get the value of an attribute using square-bracket notation: `instance[attr]`."""
        if name in self:
            return self.get(name)

        raise ApiAttributeError(
            "{0} has no attribute '{1}'".format(type(self).__name__, name), [e for e in (self._path_to_item, name) if e]
        )

    def __contains__(self, name):
        """Used by `in` operator to check if an attribute value was set in an instance: `'attr' in instance`."""
        if name in self.required_properties:
            return name in self.__dict__

        return name in self.__dict__["_data_store"]

    def to_str(self):
        """Returns the string representation of the model"""
        return str(self.value)

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, self.__class__):
            return False

        this_val = self._data_store["value"]
        that_val = other._data_store["value"]
        return this_val == that_val


class ModelNormal(OpenApiModel):
    """
    The parent class of models whose type == object in their swagger/openapi.
    """

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_spec_property_naming",
            "_path_to_item",
            "_configuration",
            "_unparsed",
        ]
    )

    def __setitem__(self, name, value):
        """Set the value of an attribute using square-bracket notation: `instance[attr] = val`."""
        if name in self.required_properties:
            self.__dict__[name] = value
            return

        self.set_attribute(name, value)

    def get(self, name, default=None):
        """Returns the value of an attribute or some default value if the attribute was not set."""
        if name in self.required_properties:
            return self.__dict__[name]

        return self.__dict__["_data_store"].get(name, default)

    def __getitem__(self, name):
        """Get the value of an attribute using square-bracket notation: `instance[attr]`."""
        if name in self:
            return self.get(name)

        raise ApiAttributeError(
            "{0} has no attribute '{1}'".format(type(self).__name__, name), [e for e in (self._path_to_item, name) if e]
        )

    def __contains__(self, name):
        """Used by `in` operator to check if an attribute value was set in an instance: `'attr' in instance`."""
        if name in self.required_properties:
            return name in self.__dict__

        return name in self.__dict__["_data_store"]

    def to_dict(self):
        """Returns the model properties as a dict"""
        return model_to_dict(self, serialize=False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, self.__class__):
            return False

        if not set(self._data_store.keys()) == set(other._data_store.keys()):
            return False
        for _var_name, this_val in self._data_store.items():
            that_val = other._data_store[_var_name]
            if this_val != that_val:
                return False
        return True

    def __init__(self, kwargs):
        super().__init__(kwargs)
        for var_name, var_value in kwargs.items():
            if (
                var_name not in self.attribute_map
                and self._configuration is not None
                and self._configuration.discard_unknown_keys
                and self.additional_properties_type is None
            ):
                if self._spec_property_naming:
                    # If it's returned from the API, store it if we need to send it back
                    self.__dict__["_data_store"][var_name] = var_value
                continue
            setattr(self, var_name, var_value)
            if not self._spec_property_naming and var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute.")


class ModelComposed(OpenApiModel):
    """
    The parent class of models whose type == object in their swagger/openapi
    and have oneOf.

    When one sets a property we use var_name_to_model_instances to store the value in
    the correct class instances + run any type checking + validation code.
    When one gets a property we use var_name_to_model_instances to get the value
    from the correct class instances.
    This allows multiple composed schemas to contain the same property with additive
    constraints on the value.

    :var _composed_schemas: Stores the oneOf classes.
    :type _composed_schemas: dict
    :var _composed_instances: Stores a list of instances of the composed schemas
        defined in _composed_schemas. When properties are accessed in the self instance,
        they are returned from the self._data_store or the data stores in the instances
        in self._composed_schemas.
    :type _composed_schemas: list
    :var _var_name_to_model_instances: Map between a variable name on self and
        the composed instances (self included) which contain that data.
    :type _var_name_to_model_instances: dict
    """

    required_properties = set(
        [
            "_data_store",
            "_check_type",
            "_spec_property_naming",
            "_path_to_item",
            "_configuration",
            "_composed_instances",
            "_var_name_to_model_instances",
            "_additional_properties_model_instances",
            "_unparsed",
        ]
    )

    def __init__(self, kwargs):
        super().__init__(kwargs)
        constant_args = {
            "_check_type": self._check_type,
            "_path_to_item": self._path_to_item,
            "_spec_property_naming": self._spec_property_naming,
            "_configuration": self._configuration,
        }
        composed_info = validate_get_composed_info(constant_args, kwargs, self)
        self._composed_instances = composed_info[0]
        self._var_name_to_model_instances = composed_info[1]
        self._additional_properties_model_instances = composed_info[2]
        self._unparsed = any(
            isinstance(composed_instance, UnparsedObject) for composed_instance in self._composed_instances
        )

    def __setitem__(self, name, value):
        """Set the value of an attribute using square-bracket notation: `instance[attr] = val`."""
        if name in self.required_properties:
            self.__dict__[name] = value
            return

        # Set attribute on composed instances
        for model_instance in self._composed_instances:
            setattr(model_instance, name, value)
        if name not in self._var_name_to_model_instances:
            # we assigned an additional property
            self.__dict__["_var_name_to_model_instances"][name] = self._composed_instances + [self]
        return None

    __unset_attribute_value__ = object()

    def get(self, name, default=None):
        """Returns the value of an attribute or some default value if the attribute was not set."""
        if name in self.required_properties:
            return self.__dict__[name]

        # get the attribute from the correct instance
        model_instances = self._var_name_to_model_instances.get(name)
        values = []
        # A composed model stores self and child (oneof) models under
        # self._var_name_to_model_instances.
        # Any property must exist in self and all model instances
        # The value stored in all model instances must be the same
        if model_instances:
            for model_instance in model_instances:
                if name in model_instance._data_store:
                    v = model_instance._data_store[name]
                    if v not in values:
                        values.append(v)
        len_values = len(values)
        if len_values == 0:
            return default
        elif len_values == 1:
            return values[0]
        elif len_values > 1:
            raise ApiValueError(
                "Values stored for property {0} in {1} differ when looking "
                "at self and self's composed instances. All values must be "
                "the same".format(name, type(self).__name__),
                [e for e in (self._path_to_item, name) if e],
            )

    def __getitem__(self, name):
        """Get the value of an attribute using square-bracket notation: `instance[attr]`."""
        value = self.get(name, self.__unset_attribute_value__)
        if value is self.__unset_attribute_value__:
            raise ApiAttributeError(
                "{0} has no attribute '{1}'".format(type(self).__name__, name),
                [e for e in (self._path_to_item, name) if e],
            )
        return value

    def __contains__(self, name):
        """Used by `in` operator to check if an attribute value was set in an instance: `'attr' in instance`."""

        if name in self.required_properties:
            return name in self.__dict__

        model_instances = self._var_name_to_model_instances.get(name, self._additional_properties_model_instances)

        if model_instances:
            for model_instance in model_instances:
                if name in model_instance._data_store:
                    return True

        return False

    def to_dict(self):
        """Returns the model properties as a dict"""
        return model_to_dict(self, serialize=False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def get_oneof_instance(self):
        """Returns the oneOf instance"""
        return self._composed_instances[0]

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, self.__class__):
            return False

        if not set(self._data_store.keys()) == set(other._data_store.keys()):
            return False
        for _var_name, this_val in self._data_store.items():
            that_val = other._data_store[_var_name]
            if this_val != that_val:
                return False
        return True


COERCION_INDEX_BY_TYPE = {
    ModelComposed: 0,
    ModelNormal: 1,
    ModelSimple: 2,
    none_type: 3,  # The type of 'None'.
    list: 4,
    dict: 5,
    float: 6,
    int: 7,
    bool: 8,
    datetime: 9,
    date: 10,
    str: 11,
    file_type: 12,  # 'file_type' is an alias for the built-in 'file' or 'io.IOBase' type.
}

# these are used to limit what type conversions we try to do
# when we have a valid type already and we want to try converting
# to another type
UPCONVERSION_TYPE_PAIRS = (
    (str, datetime),
    (str, date),
    (int, float),  # A float may be serialized as an integer, e.g. '3' is a valid serialized float.
    (list, ModelComposed),
    (dict, ModelComposed),
    (bool, ModelComposed),
    (str, ModelComposed),
    (int, ModelComposed),
    (float, ModelComposed),
    (list, ModelComposed),
    (list, ModelNormal),
    (dict, ModelNormal),
    (bool, ModelSimple),
    (str, ModelSimple),
    (int, ModelSimple),
    (float, ModelSimple),
    (list, ModelSimple),
)

COERCIBLE_TYPE_PAIRS = {
    False: (  # client instantiation of a model with client data
        # (dict, ModelComposed),
        # (list, ModelComposed),
        # (dict, ModelNormal),
        # (list, ModelNormal),
        # (str, ModelSimple),
        # (int, ModelSimple),
        # (float, ModelSimple),
        # (list, ModelSimple),
        # (str, int),
        # (str, float),
        # (str, datetime),
        # (str, date),
        # (int, str),
        # (float, str),
    ),
    True: (  # server -> client data
        (dict, ModelComposed),
        (list, ModelComposed),
        (dict, ModelNormal),
        (list, ModelNormal),
        (bool, ModelSimple),
        (str, ModelSimple),
        (int, ModelSimple),
        (float, ModelSimple),
        (list, ModelSimple),
        # (str, int),
        # (str, float),
        (str, datetime),
        (str, date),
        # (int, str),
        # (float, str),
        (str, file_type),
    ),
}


def get_simple_class(input_value):
    """Returns an input_value's simple class that we will use for type checking.

    :param input_value: The item for which we will return the simple class.
    :type input_value: class/class_instance
    """
    if isinstance(input_value, type):
        # input_value is a class
        return input_value
    elif isinstance(input_value, tuple):
        return tuple
    elif isinstance(input_value, list):
        return list
    elif isinstance(input_value, dict):
        return dict
    elif input_value is None:
        return none_type
    elif isinstance(input_value, file_type):
        return file_type
    elif isinstance(input_value, bool):
        # this must be higher than the int check because
        # isinstance(True, int) == True
        return bool
    elif isinstance(input_value, int):
        return int
    elif isinstance(input_value, datetime):
        # this must be higher than the date check because
        # isinstance(datetime_instance, date) == True
        return datetime
    elif isinstance(input_value, date):
        return date
    elif isinstance(input_value, str):
        return str
    return type(input_value)


def check_allowed_values(allowed_values, input_variable, input_values):
    """Raises an exception if the input_values are not allowed.

    :type allowed_values: set
    :param input_variable: The name of the input variable.
    :type input_variable: str
    :param input_values: The values that we are checking to see if they are in
        allowed_values.
    :type input_values: list/str/int/float/date/datetime
    """
    if isinstance(input_values, list) and not set(input_values).issubset(allowed_values):
        invalid_values = (", ".join(map(str, set(input_values) - allowed_values)),)
        raise ApiValueError(
            "Invalid values for `%s` [%s], must be a subset of [%s]"
            % (input_variable, invalid_values, ", ".join(str(v) for v in allowed_values))
        )
    elif isinstance(input_values, dict) and not set(input_values.keys()).issubset(allowed_values):
        invalid_values = ", ".join(map(str, set(input_values.keys()) - allowed_values))
        raise ApiValueError(
            "Invalid keys in `%s` [%s], must be a subset of [%s]"
            % (input_variable, invalid_values, ", ".join(str(v) for v in allowed_values))
        )
    elif not isinstance(input_values, (list, dict)) and input_values not in allowed_values:
        raise ApiValueError(
            "Invalid value for `%s` (%s), must be one of %s" % (input_variable, input_values, allowed_values)
        )


def is_json_validation_enabled(schema_keyword, configuration=None):
    """
    Returns True if JSON schema validation is enabled for the specified
    validation keyword. This can be used to skip JSON schema structural validation
    as requested in the configuration.

    :param schema_keyword: The name of a JSON schema validation keyword.
    :type schema_keyword: string
    :param configuration: The configuration instance.
    :type configuration: Configuration
    """
    return (
        configuration is None
        or not hasattr(configuration, "_disabled_client_side_validations")
        or schema_keyword not in configuration._disabled_client_side_validations
    )


def check_validations(validations, input_variable, input_values, configuration=None):
    """Raises an exception if the input_values are invalid.

    :param validations: The validation dictionary.
    :type validations: dict
    :param input_variable: The name of the input variable.
    :type input_variable: str
    :param input_values: The values that we are checking.
    :type input_values: list/str/int/float/date/datetime
    :param configuration: The configuration instance.
    :type configuration: Configuration
    """
    if input_values is None:
        return

    if (
        is_json_validation_enabled("multipleOf", configuration)
        and "multiple_of" in validations
        and isinstance(input_values, (int, float))
        and not (float(input_values) / validations["multiple_of"]).is_integer()
    ):
        # Note 'multipleOf' will be as good as the floating point arithmetic.
        raise ApiValueError(
            "Invalid value for `%s`, value must be a multiple of " "`%s`" % (input_variable, validations["multiple_of"])
        )

    if (
        is_json_validation_enabled("maxLength", configuration)
        and "max_length" in validations
        and len(input_values) > validations["max_length"]
    ):
        raise ApiValueError(
            "Invalid value for `%s`, length must be less than or equal to "
            "`%s`" % (input_variable, validations["max_length"])
        )

    if (
        is_json_validation_enabled("minLength", configuration)
        and "min_length" in validations
        and len(input_values) < validations["min_length"]
    ):
        raise ApiValueError(
            "Invalid value for `%s`, length must be greater than or equal to "
            "`%s`" % (input_variable, validations["min_length"])
        )

    if (
        is_json_validation_enabled("maxItems", configuration)
        and "max_items" in validations
        and len(input_values) > validations["max_items"]
    ):
        raise ApiValueError(
            "Invalid value for `%s`, number of items must be less than or "
            "equal to `%s`" % (input_variable, validations["max_items"])
        )

    if (
        is_json_validation_enabled("minItems", configuration)
        and "min_items" in validations
        and len(input_values) < validations["min_items"]
    ):
        raise ValueError(
            "Invalid value for `%s`, number of items must be greater than or "
            "equal to `%s`" % (input_variable, validations["min_items"])
        )

    items = ("exclusive_maximum", "inclusive_maximum", "exclusive_minimum", "inclusive_minimum")
    if any(item in validations for item in items):
        if isinstance(input_values, list):
            max_val = max(input_values)
            min_val = min(input_values)
        elif isinstance(input_values, dict):
            max_val = max(input_values.values())
            min_val = min(input_values.values())
        else:
            max_val = input_values
            min_val = input_values

    if (
        is_json_validation_enabled("exclusiveMaximum", configuration)
        and "exclusive_maximum" in validations
        and max_val >= validations["exclusive_maximum"]
    ):
        raise ApiValueError(
            "Invalid value for `%s`, must be a value less than `%s`"
            % (input_variable, validations["exclusive_maximum"])
        )

    if (
        is_json_validation_enabled("maximum", configuration)
        and "inclusive_maximum" in validations
        and max_val > validations["inclusive_maximum"]
    ):
        raise ApiValueError(
            "Invalid value for `%s`, must be a value less than or equal to "
            "`%s`" % (input_variable, validations["inclusive_maximum"])
        )

    if (
        is_json_validation_enabled("exclusiveMinimum", configuration)
        and "exclusive_minimum" in validations
        and min_val <= validations["exclusive_minimum"]
    ):
        raise ApiValueError(
            "Invalid value for `%s`, must be a value greater than `%s`"
            % (input_variable, validations["exclusive_maximum"])
        )

    if (
        is_json_validation_enabled("minimum", configuration)
        and "inclusive_minimum" in validations
        and min_val < validations["inclusive_minimum"]
    ):
        raise ApiValueError(
            "Invalid value for `%s`, must be a value greater than or equal "
            "to `%s`" % (input_variable, validations["inclusive_minimum"])
        )
    flags = validations.get("regex", {}).get("flags", 0)
    if (
        is_json_validation_enabled("pattern", configuration)
        and "regex" in validations
        and not re.search(validations["regex"]["pattern"], input_values, flags=flags)
    ):
        err_msg = r"Invalid value for `%s`, must match regular expression `%s`" % (
            input_variable,
            validations["regex"]["pattern"],
        )
        if flags != 0:
            # Don't print the regex flags if the flags are not
            # specified in the OAS document.
            err_msg = r"%s with flags=`%s`" % (err_msg, flags)
        raise ApiValueError(err_msg)


def order_response_types(required_types):
    """Returns the required types sorted in coercion order.

    :param required_types: Collection of classes or instance of
        list or dict with class information inside it.
    :type required_types: list/tuple

    :return: Coercion order sorted collection of classes or instance
        of list or dict with class information inside it.
    :rtype: list
    """

    def index_getter(class_or_instance):
        if isinstance(class_or_instance, list):
            return COERCION_INDEX_BY_TYPE[list]
        elif isinstance(class_or_instance, dict):
            return COERCION_INDEX_BY_TYPE[dict]
        elif inspect.isclass(class_or_instance) and issubclass(class_or_instance, ModelComposed):
            return COERCION_INDEX_BY_TYPE[ModelComposed]
        elif inspect.isclass(class_or_instance) and issubclass(class_or_instance, ModelNormal):
            return COERCION_INDEX_BY_TYPE[ModelNormal]
        elif inspect.isclass(class_or_instance) and issubclass(class_or_instance, ModelSimple):
            return COERCION_INDEX_BY_TYPE[ModelSimple]
        elif class_or_instance in COERCION_INDEX_BY_TYPE:
            return COERCION_INDEX_BY_TYPE[class_or_instance]
        raise ApiValueError("Unsupported type: %s" % class_or_instance)

    sorted_types = sorted(required_types, key=index_getter)
    return sorted_types


def remove_uncoercible(required_types_classes, current_item, spec_property_naming, must_convert=True):
    """Only keeps the type conversions that are possible.

    :param required_types_classes: Classes that are required, these should be
        ordered by COERCION_INDEX_BY_TYPE.
    :type required_types_classes: tuple
    :param spec_property_naming: True if the variable names in the input data
        are serialized names as specified in the OpenAPI document.  False if the
        variables names in the input data are python variable names in PEP-8 snake
        case.
    :type spec_property_naming: bool
    :param current_item: The current item (input data) to be converted.

    :param must_convert: If True the item to convert is of the wrong type and
        we want a big list of coercibles if False, we want a limited list of coercibles.
    :type must_convert: bool

    :return: The remaining coercible required types, classes only.
    :rtype: list
    """
    current_type_simple = get_simple_class(current_item)

    results_classes = []
    for required_type_class in required_types_classes:
        # convert our models to OpenApiModel
        required_type_class_simplified = required_type_class
        if isinstance(required_type_class_simplified, type):
            if issubclass(required_type_class_simplified, ModelComposed):
                required_type_class_simplified = ModelComposed
            elif issubclass(required_type_class_simplified, ModelNormal):
                required_type_class_simplified = ModelNormal
            elif issubclass(required_type_class_simplified, ModelSimple):
                required_type_class_simplified = ModelSimple

        if required_type_class_simplified == current_type_simple:
            # don't consider converting to one's own class
            continue

        class_pair = (current_type_simple, required_type_class_simplified)
        if must_convert and class_pair in COERCIBLE_TYPE_PAIRS[spec_property_naming]:
            results_classes.append(required_type_class)
        elif class_pair in UPCONVERSION_TYPE_PAIRS:
            results_classes.append(required_type_class)
    return results_classes


def get_possible_classes(cls, from_server_context):
    possible_classes = [cls]
    if from_server_context:
        return possible_classes
    if issubclass(cls, ModelComposed):
        possible_classes.extend(composed_model_input_classes(cls))
    return possible_classes


def get_required_type_classes(required_types_mixed, spec_property_naming):
    """Converts the tuple required_types into a tuple and a dict described below.

    :param required_types_mixed: Will contain either classes or instance of
        list or dict.
    :type required_types_mixed: tuple/list
    :param spec_property_naming: if True these values came from the server, and
        we use the data types in our endpoints.  If False, we are client side and we
        need to include oneOf classes inside the data types in our endpoints.
    :type spec_property_naming: bool

    :return (valid_classes, dict_valid_class_to_child_types_mixed):
        valid_classes (tuple): The valid classes that the current item should be.
        dict_valid_class_to_child_types_mixed (dict):

            valid_class (class): This is the key.
            child_types_mixed (list/dict/tuple): Describes the valid child types.

    :rtype: tuple
    """
    valid_classes = []
    child_req_types_by_current_type = {}
    for required_type in required_types_mixed:
        if isinstance(required_type, list):
            valid_classes.append(list)
            child_req_types_by_current_type[list] = required_type
        elif isinstance(required_type, tuple):
            valid_classes.append(tuple)
            child_req_types_by_current_type[tuple] = required_type
        elif isinstance(required_type, dict):
            valid_classes.append(dict)
            child_req_types_by_current_type[dict] = required_type[str]
        else:
            valid_classes.extend(get_possible_classes(required_type, spec_property_naming))
    return tuple(valid_classes), child_req_types_by_current_type


def change_keys_js_to_python(input_dict, model_class):
    """
    Converts from javascript_key keys in the input_dict to python_keys in
    the output dict using the mapping in model_class.
    If the input_dict contains a key which does not declared in the model_class,
    the key is added to the output dict as is. The assumption is the model_class
    may have undeclared properties (additionalProperties attribute in the OAS
    document).
    """
    if issubclass(model_class, ModelComposed):
        attribute_map = {}
        for t in model_class._composed_schemas.get("oneOf", ()):
            if issubclass(t, OpenApiModel):
                attribute_map.update(t.attribute_map)
    elif not getattr(model_class, "attribute_map", None):
        return input_dict
    else:
        attribute_map = model_class.attribute_map
    output_dict = {}
    reversed_attr_map = {value: key for key, value in attribute_map.items()}
    for javascript_key, value in input_dict.items():
        python_key = reversed_attr_map.get(javascript_key)
        if python_key is None:
            # if the key is unknown, it is in error or it is an
            # additionalProperties variable
            python_key = javascript_key
        output_dict[python_key] = value
    return output_dict


def get_type_error(var_value, path_to_item, valid_classes, key_type=False):
    error_msg = type_error_message(
        var_name=path_to_item[-1], var_value=var_value, valid_classes=valid_classes, key_type=key_type
    )
    return ApiTypeError(error_msg, path_to_item=path_to_item, valid_classes=valid_classes, key_type=key_type)


def deserialize_primitive(data, klass, path_to_item):
    """Deserializes string to primitive type.

    :type data: str/int/float
    :param klass: The class to convert to.
    :type klass: str/class

    :rtype: int, float, str, bool, date, datetime
    """
    additional_message = ""
    try:
        if klass in {datetime, date}:
            additional_message = (
                "If you need your parameter to have a fallback "
                "string value, please set its type as `type: {}` in your "
                "spec. That allows the value to be any type. "
            )
            if klass == datetime:
                if len(data) < 8:
                    raise ValueError("This is not a datetime")
                # The string should be in iso8601 datetime format.
                parsed_datetime = parse(data)
                date_only = (
                    parsed_datetime.hour == parsed_datetime.minute == parsed_datetime.second == 0
                    and parsed_datetime.tzinfo is None
                    and 8 <= len(data) <= 10
                )
                if date_only:
                    raise ValueError("This is a date, not a datetime")
                return parsed_datetime
            elif klass == date:
                if len(data) < 8:
                    raise ValueError("This is not a date")
                return parse(data).date()
        else:
            converted_value = klass(data)
            if isinstance(data, str) and klass == float:
                if str(converted_value) != data:
                    # '7' -> 7.0 -> '7.0' != '7'
                    raise ValueError("This is not a float")
            return converted_value
    except (OverflowError, ValueError) as ex:
        # parse can raise OverflowError
        raise ApiValueError(
            "{0}Failed to parse {1} as {2}".format(additional_message, repr(data), klass.__name__),
            path_to_item=path_to_item,
        ) from ex


def deserialize_model(model_data, model_class, path_to_item, check_type, configuration, spec_property_naming):
    """Deserializes model_data to model instance.

    :param model_data: Data to instantiate the model.
    :type model_data: int/str/float/bool/none_type/list/dict
    :param model_class: The model class.
    :type model_class: OpenApiModel
    :param path_to_item: Path to the model in the received data.
    :type path_to_item: list
    :param check_type: Whether to check the data tupe for the values in
        the model.
    :type check_type: bool
    :param configuration: The instance to use to convert files.
    :type configuration: Configuration
    :param spec_property_naming: True if the variable names in the input
        data are serialized names as specified in the OpenAPI document.
        False if the variables names in the input data are python
        variable names in PEP-8 snake case.
    :type spec_property_naming: bool

    :return: The model instance.
    """

    kw_args = dict(
        _check_type=check_type,
        _path_to_item=path_to_item,
        _configuration=configuration,
        _spec_property_naming=spec_property_naming,
    )

    if issubclass(model_class, ModelSimple):
        return model_class._new_from_openapi_data(model_data, **kw_args)
    elif isinstance(model_data, list):
        if issubclass(model_class, ModelComposed) and allows_single_value_input(model_class):
            return model_class._new_from_openapi_data(model_data, **kw_args)
        else:
            return model_class._new_from_openapi_data(*model_data, **kw_args)
    if isinstance(model_data, dict):
        kw_args.update(model_data)
        return model_class._new_from_openapi_data(**kw_args)
    elif isinstance(model_data, PRIMITIVE_TYPES):
        return model_class._new_from_openapi_data(model_data, **kw_args)


def deserialize_file(response_data, temp_folder_path, content_disposition=None):
    """Deserializes body to file.

    Saves response body into a file in a temporary folder, using the filename
    from the `Content-Disposition` header if provided.

    :param response_data: The file data to write.
    :type response_data: str
    :param temp_folder_path: The directory in which the client creates temporary files.
    :type temp_folder_path: str
    :param content_disposition: The value of the Content-Disposition
        header.
    :type content_disposition: str

    :return: The deserialized file which is open. The user is responsible for
        closing and reading the file.
    :rtype: file_type
    """
    fd, path = tempfile.mkstemp(dir=temp_folder_path)
    os.close(fd)
    os.remove(path)

    if content_disposition:
        filename = re.search(r'filename=[\'"]?([^\'"\s]+)[\'"]?', content_disposition).group(1)
        path = os.path.join(os.path.dirname(path), filename)

    with open(path, "wb") as f:
        if isinstance(response_data, str):
            # change str to bytes so we can write it
            response_data = response_data.encode("utf-8")
        f.write(response_data)

    f = open(path, "rb")
    return f


def attempt_convert_item(
    input_value,
    valid_classes,
    path_to_item,
    configuration,
    spec_property_naming,
    key_type=False,
    must_convert=False,
    check_type=True,
):
    """
    :param input_value: The data to convert.
    :param valid_classes: The classes that are valid.
    :param path_to_item: The path to the item to convert.
    :type path_to_item: list
    :param configuration: The instance to use to convert files.
    :type configuration: Configuration
    :param spec_property_naming: True if the variable names in the input data
        are serialized names as specified in the OpenAPI document.  False if the
        variables names in the input data are python variable names in PEP-8 snake
        case.
    :type spec_property_naming: bool
    :param key_type: If True we need to convert a key type (not supported)
    :type key_type: bool
    :param must_convert: If True we must convert.
    :type must_convert: bool
    :param check_type: If True we check the type or the returned data in
        ModelComposed/ModelNormal/ModelSimple instances.
    :type check_type: bool
    """
    valid_classes_ordered = order_response_types(valid_classes)
    valid_classes_coercible = remove_uncoercible(valid_classes_ordered, input_value, spec_property_naming)
    if not valid_classes_coercible or key_type:
        # we do not handle keytype errors, json will take care
        # of this for us
        if configuration is None or not configuration.discard_unknown_keys:
            raise get_type_error(input_value, path_to_item, valid_classes, key_type=key_type)
    for valid_class in valid_classes_coercible:
        try:
            if issubclass(valid_class, OpenApiModel):
                return deserialize_model(
                    input_value, valid_class, path_to_item, check_type, configuration, spec_property_naming
                )
            elif valid_class == file_type:
                return deserialize_file(input_value, configuration.temp_folder_path)
            return deserialize_primitive(input_value, valid_class, path_to_item)
        except (ApiTypeError, ApiValueError, ApiKeyError) as conversion_exc:
            if must_convert:
                raise conversion_exc
            # if we have conversion errors when must_convert == False
            # we ignore the exception and move on to the next class
            continue
    if must_convert:
        raise get_type_error(input_value, path_to_item, valid_classes, key_type=key_type)
    # we were unable to convert, must_convert == False
    return input_value


def is_type_nullable(input_type):
    """
    Returns True if None is an allowed value for the specified input_type.

    A type is nullable if at least one of the following conditions is True:

        1. The OAS 'nullable' attribute has been specified,
        2. The type is the 'null' type,
        3. The type is a oneOf composed schema, and a child schema is
           the 'null' type.

    :param input_type: The class of the input_value that we are checking.
    :type input_type: type

    :rtype: bool
    """
    if input_type is none_type:
        return True
    if issubclass(input_type, OpenApiModel) and input_type._nullable:
        return True
    if issubclass(input_type, ModelComposed):
        # If oneOf, check if the 'null' type is one of the allowed types.
        for t in input_type._composed_schemas.get("oneOf", ()):
            if is_type_nullable(t):
                return True
    return False


def is_valid_type(input_class_simple, valid_classes):
    """
    :param input_class_simple: The class of the input_value that we are checking.
    :type input_class_simple: class:
    :param valid_classes: The valid classes that the current item should be.
    :type valid_classes: tuple

    :rtype: bool
    """
    valid_type = input_class_simple in valid_classes
    if not valid_type and (issubclass(input_class_simple, OpenApiModel) or input_class_simple is none_type):
        for valid_class in valid_classes:
            if input_class_simple is none_type and is_type_nullable(valid_class):
                # Schema is oneOf and the 'null' type is one of the allowed types.
                return True
            if issubclass(valid_class, OpenApiModel):
                continue
    return valid_type


def validate_and_convert_types(
    input_value, required_types_mixed, path_to_item, spec_property_naming, check_type, configuration=None
):
    """Raises a TypeError is there is a problem, otherwise returns value.

    :param input_value: The data to validate/convert.
    :param required_types_mixed: A list of valid classes, or a list tuples of
        valid classes, or a dict where the value is a tuple of value classes.
    :type required_types_mixed: list/dict/tuple
    :param path_to_item: The path to the data being validated this stores a
        list of keys or indices to get to the data being validated.
    :type path_to_item: list
    :param spec_property_naming: True if the variable names in the input
        data are serialized names as specified in the OpenAPI document.  False
        if the variables names in the input data are python variable names in PEP-8
        snake case.
    :type spec_property_naming: bool
    :param check_type: If True, type will be checked and conversion
        will be attempted.
    :type check_type: bool
    :param configuration:: The configuration class to use when converting
        file_type items.
    :type configuration: Configuration

    :return: The correctly typed value.

    :raise: ApiTypeError
    """
    results = get_required_type_classes(required_types_mixed, spec_property_naming)
    valid_classes, child_req_types_by_current_type = results

    input_class_simple = get_simple_class(input_value)
    valid_type = is_valid_type(input_class_simple, valid_classes)
    if not valid_type:
        # if input_value is not valid_type try to convert it
        return attempt_convert_item(
            input_value,
            valid_classes,
            path_to_item,
            configuration,
            spec_property_naming,
            must_convert=True,
            check_type=check_type,
        )

    # input_value's type is in valid_classes
    if len(valid_classes) > 1 and configuration:
        # there are valid classes which are not the current class
        valid_classes_coercible = remove_uncoercible(
            valid_classes, input_value, spec_property_naming, must_convert=False
        )
        if valid_classes_coercible:
            return attempt_convert_item(
                input_value,
                valid_classes_coercible,
                path_to_item,
                configuration,
                spec_property_naming,
                check_type=check_type,
            )

    if child_req_types_by_current_type == {}:
        # all types are of the required types and there are no more inner
        # variables left to look at
        return input_value
    inner_required_types = child_req_types_by_current_type.get(type(input_value))
    if inner_required_types is None:
        # for this type, there are not more inner variables left to look at
        return input_value
    if isinstance(input_value, list):
        if input_value == []:
            # allow an empty list
            return input_value
        result = []
        for index, inner_value in enumerate(input_value):
            inner_path = list(path_to_item)
            inner_path.append(index)
            result.append(
                validate_and_convert_types(
                    inner_value,
                    inner_required_types,
                    inner_path,
                    spec_property_naming,
                    check_type,
                    configuration=configuration,
                )
            )
        return result
    elif isinstance(input_value, dict):
        if input_value == {}:
            # allow an empty dict
            return input_value
        result = {}
        for inner_key, inner_val in input_value.items():
            inner_path = list(path_to_item)
            inner_path.append(inner_key)
            if get_simple_class(inner_key) != str:
                raise get_type_error(inner_key, inner_path, valid_classes, key_type=True)
            result[inner_key] = validate_and_convert_types(
                inner_val,
                inner_required_types,
                inner_path,
                spec_property_naming,
                check_type,
                configuration=configuration,
            )
        return result
    return input_value


def model_to_dict(model_instance, serialize=True):
    """Returns the model properties as a dict.

    :param model_instance: The model instance that will be converted to a dict.
    :param serialize: If True, the keys in the dict will be values from
        attribute_map.
    :type serialize: bool
    """
    result = {}

    model_instances = [model_instance]
    if model_instance._composed_schemas:
        model_instances.extend(model_instance._composed_instances)
    seen_json_attribute_names = set()
    used_fallback_python_attribute_names = set()
    py_to_json_map = {}
    for model_instance in model_instances:
        for attr, value in model_instance._data_store.items():
            if serialize:
                # we use get here because additional property key names do not
                # exist in attribute_map
                try:
                    attr = model_instance.attribute_map[attr]
                    py_to_json_map.update(model_instance.attribute_map)
                    seen_json_attribute_names.add(attr)
                except KeyError:
                    used_fallback_python_attribute_names.add(attr)
            if isinstance(value, list):
                if not value:
                    # empty list or None
                    result[attr] = value
                else:
                    res = []
                    for v in value:
                        if isinstance(v, PRIMITIVE_TYPES) or v is None:
                            res.append(v)
                        elif isinstance(v, ModelSimple):
                            res.append(v.value)
                        elif isinstance(v, OpenApiModel):
                            res.append(model_to_dict(v, serialize=serialize))
                        else:
                            res.append(v)
                    result[attr] = res
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], model_to_dict(item[1], serialize=serialize))
                        if hasattr(item[1], "_data_store")
                        else item,
                        value.items(),
                    )
                )
            elif isinstance(value, ModelSimple):
                result[attr] = value.value
            elif hasattr(value, "_data_store"):
                result[attr] = model_to_dict(value, serialize=serialize)
            else:
                result[attr] = value
    if serialize:
        for python_key in used_fallback_python_attribute_names:
            json_key = py_to_json_map.get(python_key)
            if json_key is None:
                continue
            if python_key == json_key:
                continue
            json_key_assigned_no_need_for_python_key = json_key in seen_json_attribute_names
            if json_key_assigned_no_need_for_python_key:
                del result[python_key]

    return result


def type_error_message(var_value=None, var_name=None, valid_classes=None, key_type=None):
    """
    :param var_value: The variable which has the type error.
    :param var_name: The name of the variable which has the type error.
    :type var_name: str
    :param valid_classes: The accepted classes for current_item's value.
    :type valid_classes: tuple
    :param key_type: False if our value is a value in a dict True if it is a
        key in a dict False if our item is an item in a list.
    :type key_type: bool
    """
    key_or_value = "value"
    if key_type:
        key_or_value = "key"
    valid_classes_phrase = get_valid_classes_phrase(valid_classes)
    msg = "Invalid type for variable '{0}'. Required {1} type {2} and " "passed type was {3}".format(
        var_name,
        key_or_value,
        valid_classes_phrase,
        type(var_value).__name__,
    )
    return msg


def get_valid_classes_phrase(input_classes):
    """Returns a string phrase describing what types are allowed."""
    all_class_names = [cls.__name__ for cls in input_classes]
    all_class_names.sort()
    if len(all_class_names) == 1:
        return "is {0}".format(all_class_names[0])
    return "is one of [{0}]".format(", ".join(all_class_names))


def get_oneof_instance(cls, model_kwargs, constant_kwargs, model_arg=None):
    """
    Find the oneOf schema that matches the input data (e.g. payload).
    If exactly one schema matches the input data, an instance of that schema
    is returned.
    If zero or more than one schema match the input data, an exception is raised.
    In OAS 3.x, the payload MUST, by validation, match exactly one of the
    schemas described by oneOf.

    :param cls: The class we are handling.
    :param model_kwargs: var_name to var_value.
        The input data, e.g. the payload that must match a oneOf schema
        in the OpenAPI document.
    :type model_kwargs: dict
    :param constant_kwargs: var_name to var_value
        args that every model requires, including configuration, server
        and path to item.
    :type constant_kwargs: dict
    :param model_arg: The value to assign to a primitive class or ModelSimple class.
        Notes:
        - this is only passed in when oneOf includes types which are not object
        - None is used to suppress handling of model_arg, nullable models are handled in __new__
    :type model_arg: int, float, bool, str, date, datetime, ModelSimple, None
    """
    if len(cls._composed_schemas["oneOf"]) == 0:
        return None

    oneof_instances = []
    # Iterate over each oneOf schema and determine if the input data
    # matches the oneOf schemas.
    for oneof_class in cls._composed_schemas["oneOf"]:
        # The composed oneOf schema allows the 'null' type and the input data
        # is the null value. This is a OAS >= 3.1 feature.
        if oneof_class is none_type:
            # skip none_types because we are deserializing dict data.
            # none_type deserialization is handled in the __new__ method
            continue

        single_value_input = allows_single_value_input(oneof_class)

        with suppress(Exception):
            if not single_value_input:
                if constant_kwargs.get("_spec_property_naming"):
                    oneof_instance = oneof_class(
                        **change_keys_js_to_python(model_kwargs, oneof_class), **constant_kwargs
                    )
                else:
                    oneof_instance = oneof_class(**model_kwargs, **constant_kwargs)
                if not oneof_instance._unparsed:
                    oneof_instances.append(oneof_instance)
            else:
                if issubclass(oneof_class, ModelSimple):
                    oneof_instance = oneof_class(model_arg, **constant_kwargs)
                    if not oneof_instance._unparsed:
                        oneof_instances.append(oneof_instance)
                elif oneof_class in PRIMITIVE_TYPES:
                    oneof_instance = validate_and_convert_types(
                        model_arg,
                        (oneof_class,),
                        constant_kwargs.get("_path_to_item", ()),
                        constant_kwargs.get("_spec_property_naming", False),
                        constant_kwargs.get("_check_type", True),
                        configuration=constant_kwargs.get("_configuration"),
                    )
                    oneof_instances.append(oneof_instance)
    if len(oneof_instances) != 1:
        return UnparsedObject(**model_kwargs)
    return oneof_instances[0]


def get_discarded_args(self, composed_instances, model_args):
    """
    Gathers the args that were discarded by configuration.discard_unknown_keys
    """
    model_arg_keys = model_args.keys()
    discarded_args = set()
    # arguments passed to self were already converted to python names
    # before __init__ was called
    for instance in composed_instances:
        all_keys = set(model_to_dict(instance, serialize=False).keys())
        js_keys = model_to_dict(instance).keys()
        all_keys.update(js_keys)
        discarded_keys = model_arg_keys - all_keys
        discarded_args.update(discarded_keys)
    return discarded_args


def validate_get_composed_info(constant_args, model_args, self):
    """
    For composed schemas, generate schema instances for all schemas in the
    oneOf definition. If additional properties are allowed, also assign
    those properties on all matched schemas that contain additionalProperties.
    Openapi schemas are python classes.

    Exceptions are raised if:
    - 0 or > 1 oneOf schema matches the model_args input data

    :param constant_args: These are the args that every model requires.
    :type constant_args: dict
    :param model_args: These are the required and optional spec args that
        were passed in to make this model.
    :type model_args: dict
    :param self: The class that we are instantiating.  This class contains
        self._composed_schemas.
    :type self: class

    :return:
        composed_instances (list): the composed instances which are not
            self
        var_name_to_model_instances (dict): a dict going from var_name
            to the model_instance which holds that var_name
            the model_instance may be self or an instance of one of the
            classes in self.composed_instances()
        additional_properties_model_instances (list): a list of the
            model instances which have the property
            additional_properties_type. This list can include self
    :rtype: list
    """
    # Create composed_instances
    composed_instances = []
    oneof_instance = get_oneof_instance(self.__class__, model_args, constant_args)
    if oneof_instance is not None:
        composed_instances.append(oneof_instance)

    additional_properties_model_instances = []
    if self.additional_properties_type is not None:
        additional_properties_model_instances = [self]

    discarded_args = get_discarded_args(self, composed_instances, model_args)

    # Map variable names to composed_instances
    var_name_to_model_instances = {}
    for prop_name in model_args:
        if prop_name not in discarded_args:
            var_name_to_model_instances[prop_name] = [self] + composed_instances

    return [composed_instances, var_name_to_model_instances, additional_properties_model_instances, discarded_args]


class UnparsedObject(ModelNormal):
    """A model for an oneOf we don't know about."""

    required_properties = set(
        [
            "_data_store",
            "_unparsed",
        ]
    )

    def __init__(self, **kwargs):

        self._data_store = {}
        self._unparsed = True

        for var_name, var_value in kwargs.items():
            self.__dict__[var_name] = var_value
            self.__dict__["_data_store"][var_name] = var_value


def get_attribute_from_path(obj, path, default=None):
    """Return an attribute at `path` from the passed object."""
    for elt in path.split("."):
        try:
            obj = obj[elt]
        except (KeyError, AttributeError):
            if default is None:
                raise
            return default
    return obj


def set_attribute_from_path(obj, path, value, params_map):
    """Set an attribute at `path` with the given value."""
    elts = path.split(".")
    last = elts.pop(-1)
    root = None
    for i, elt in enumerate(elts):
        if i:
            root = root.openapi_types[elt][0]
        else:
            root = params_map[elt]["openapi_types"][0]
        try:
            obj = obj[elt]
        except (KeyError, AttributeError):
            obj = root()
    obj[last] = value



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.security_monitoring_triage_user import SecurityMonitoringTriageUser


class SecurityMonitoringSignalAssigneeUpdateAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.security_monitoring_triage_user import SecurityMonitoringTriageUser

        return {
            "assignee": (SecurityMonitoringTriageUser,),
            "version": (int,),
        }

    attribute_map = {
        "assignee": "assignee",
        "version": "version",
    }

    def __init__(self_, assignee: SecurityMonitoringTriageUser, version: Union[int, UnsetType] = unset, **kwargs):
        """
        Attributes describing the new assignee of a security signal.

        :param assignee: Object representing a given user entity.
        :type assignee: SecurityMonitoringTriageUser

        :param version: Version of the updated signal. If server side version is higher, update will be rejected.
        :type version: int, optional
        """
        if version is not unset:
            kwargs["version"] = version
        super().__init__(kwargs)

        self_.assignee = assignee



"""
Create a new dashboard with timeseries widget containing style attributes
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.timeseries_widget_definition import TimeseriesWidgetDefinition
from datadog_api_client.v1.model.timeseries_widget_definition_type import TimeseriesWidgetDefinitionType
from datadog_api_client.v1.model.timeseries_widget_request import TimeseriesWidgetRequest
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_display_type import WidgetDisplayType
from datadog_api_client.v1.model.widget_line_type import WidgetLineType
from datadog_api_client.v1.model.widget_line_width import WidgetLineWidth
from datadog_api_client.v1.model.widget_request_style import WidgetRequestStyle

body = Dashboard(
    layout_type=DashboardLayoutType.ORDERED,
    title="Example-Create_a_new_dashboard_with_timeseries_widget_containing_style_attributes with timeseries widget",
    widgets=[
        Widget(
            definition=TimeseriesWidgetDefinition(
                type=TimeseriesWidgetDefinitionType.TIMESERIES,
                requests=[
                    TimeseriesWidgetRequest(
                        q="sum:trace.test.errors{env:prod,service:datadog-api-spec} by {resource_name}.as_count()",
                        on_right_yaxis=False,
                        style=WidgetRequestStyle(
                            palette="warm",
                            line_type=WidgetLineType.SOLID,
                            line_width=WidgetLineWidth.NORMAL,
                        ),
                        display_type=WidgetDisplayType.BARS,
                    ),
                ],
            ),
        ),
    ],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.relationship_to_incident_attachment import RelationshipToIncidentAttachment
    from datadog_api_client.v2.model.nullable_relationship_to_user import NullableRelationshipToUser
    from datadog_api_client.v2.model.relationship_to_user import RelationshipToUser
    from datadog_api_client.v2.model.relationship_to_incident_integration_metadatas import (
        RelationshipToIncidentIntegrationMetadatas,
    )


class IncidentResponseRelationships(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.relationship_to_incident_attachment import RelationshipToIncidentAttachment
        from datadog_api_client.v2.model.nullable_relationship_to_user import NullableRelationshipToUser
        from datadog_api_client.v2.model.relationship_to_user import RelationshipToUser
        from datadog_api_client.v2.model.relationship_to_incident_integration_metadatas import (
            RelationshipToIncidentIntegrationMetadatas,
        )

        return {
            "attachments": (RelationshipToIncidentAttachment,),
            "commander_user": (NullableRelationshipToUser,),
            "created_by_user": (RelationshipToUser,),
            "integrations": (RelationshipToIncidentIntegrationMetadatas,),
            "last_modified_by_user": (RelationshipToUser,),
        }

    attribute_map = {
        "attachments": "attachments",
        "commander_user": "commander_user",
        "created_by_user": "created_by_user",
        "integrations": "integrations",
        "last_modified_by_user": "last_modified_by_user",
    }

    def __init__(
        self_,
        attachments: Union[RelationshipToIncidentAttachment, UnsetType] = unset,
        commander_user: Union[NullableRelationshipToUser, UnsetType] = unset,
        created_by_user: Union[RelationshipToUser, UnsetType] = unset,
        integrations: Union[RelationshipToIncidentIntegrationMetadatas, UnsetType] = unset,
        last_modified_by_user: Union[RelationshipToUser, UnsetType] = unset,
        **kwargs,
    ):
        """
        The incident's relationships from a response.

        :param attachments: A relationship reference for attachments.
        :type attachments: RelationshipToIncidentAttachment, optional

        :param commander_user: Relationship to user.
        :type commander_user: NullableRelationshipToUser, optional

        :param created_by_user: Relationship to user.
        :type created_by_user: RelationshipToUser, optional

        :param integrations: A relationship reference for multiple integration metadata objects.
        :type integrations: RelationshipToIncidentIntegrationMetadatas, optional

        :param last_modified_by_user: Relationship to user.
        :type last_modified_by_user: RelationshipToUser, optional
        """
        if attachments is not unset:
            kwargs["attachments"] = attachments
        if commander_user is not unset:
            kwargs["commander_user"] = commander_user
        if created_by_user is not unset:
            kwargs["created_by_user"] = created_by_user
        if integrations is not unset:
            kwargs["integrations"] = integrations
        if last_modified_by_user is not unset:
            kwargs["last_modified_by_user"] = last_modified_by_user
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetComparator(ModelSimple):
    """
    Comparator to apply.

    :param value: Must be one of [">", ">=", "<", "<="].
    :type value: str
    """

    allowed_values = {
        ">",
        ">=",
        "<",
        "<=",
    }
    GREATER_THAN: ClassVar["WidgetComparator"]
    GREATER_THAN_OR_EQUAL_TO: ClassVar["WidgetComparator"]
    LESS_THAN: ClassVar["WidgetComparator"]
    LESS_THAN_OR_EQUAL_TO: ClassVar["WidgetComparator"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetComparator.GREATER_THAN = WidgetComparator(">")
WidgetComparator.GREATER_THAN_OR_EQUAL_TO = WidgetComparator(">=")
WidgetComparator.LESS_THAN = WidgetComparator("<")
WidgetComparator.LESS_THAN_OR_EQUAL_TO = WidgetComparator("<=")



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_basic_auth_oauth_token_api_authentication import (
        SyntheticsBasicAuthOauthTokenApiAuthentication,
    )
    from datadog_api_client.v1.model.synthetics_basic_auth_oauth_client_type import SyntheticsBasicAuthOauthClientType


class SyntheticsBasicAuthOauthClient(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_basic_auth_oauth_token_api_authentication import (
            SyntheticsBasicAuthOauthTokenApiAuthentication,
        )
        from datadog_api_client.v1.model.synthetics_basic_auth_oauth_client_type import (
            SyntheticsBasicAuthOauthClientType,
        )

        return {
            "access_token_url": (str,),
            "audience": (str,),
            "client_id": (str,),
            "client_secret": (str,),
            "resource": (str,),
            "scope": (str,),
            "token_api_authentication": (SyntheticsBasicAuthOauthTokenApiAuthentication,),
            "type": (SyntheticsBasicAuthOauthClientType,),
        }

    attribute_map = {
        "access_token_url": "accessTokenUrl",
        "audience": "audience",
        "client_id": "clientId",
        "client_secret": "clientSecret",
        "resource": "resource",
        "scope": "scope",
        "token_api_authentication": "tokenApiAuthentication",
        "type": "type",
    }

    def __init__(
        self_,
        access_token_url: str,
        client_id: str,
        client_secret: str,
        token_api_authentication: SyntheticsBasicAuthOauthTokenApiAuthentication,
        audience: Union[str, UnsetType] = unset,
        resource: Union[str, UnsetType] = unset,
        scope: Union[str, UnsetType] = unset,
        type: Union[SyntheticsBasicAuthOauthClientType, UnsetType] = unset,
        **kwargs,
    ):
        """
        Object to handle ``oauth client`` authentication when performing the test.

        :param access_token_url: Access token URL to use when performing the authentication.
        :type access_token_url: str

        :param audience: Audience to use when performing the authentication.
        :type audience: str, optional

        :param client_id: Client ID to use when performing the authentication.
        :type client_id: str

        :param client_secret: Client secret to use when performing the authentication.
        :type client_secret: str

        :param resource: Resource to use when performing the authentication.
        :type resource: str, optional

        :param scope: Scope to use when performing the authentication.
        :type scope: str, optional

        :param token_api_authentication: Type of token to use when performing the authentication.
        :type token_api_authentication: SyntheticsBasicAuthOauthTokenApiAuthentication

        :param type: The type of basic authentication to use when performing the test.
        :type type: SyntheticsBasicAuthOauthClientType, optional
        """
        if audience is not unset:
            kwargs["audience"] = audience
        if resource is not unset:
            kwargs["resource"] = resource
        if scope is not unset:
            kwargs["scope"] = scope
        if type is not unset:
            kwargs["type"] = type
        super().__init__(kwargs)

        self_.access_token_url = access_token_url
        self_.client_id = client_id
        self_.client_secret = client_secret
        self_.token_api_authentication = token_api_authentication
